// ==========================================================================
//
// Paint.cpp	Painting code for Windows test application
//
// --------------------------------------------------------------------------
//
// 10-15-2003	Hans-Martin Will	initial version
//
// ==========================================================================


#include "stdafx.h"
#include "UIHelper.h"

// --------------------------------------------------------------------------
// Avoid name conflict with GdiPlus
// --------------------------------------------------------------------------
#ifdef PixelFormatAlpha
#undef PixelFormatAlpha
#endif

#include "fixed.h"

#include "test.h"
#include "GLES/egl.h"
#include "GLES/gl.h"
#include "GLES/glext.h"
//#include "Color.h"

#define USE_BUFFERS 0

GLAPI EGLBoolean APIENTRY eglSaveSurfaceHM(EGLSurface surface, const TCHAR * filename);

#ifndef _WIN32_WCE
using namespace Gdiplus;

namespace {
	bool LoadTexture(const WCHAR *filename) {
		//
		Bitmap* bitmap = new Bitmap(filename);

		if (!bitmap)
			return false;

		BitmapData* bitmapData = new BitmapData;
		Rect rect(0, 0, bitmap->GetWidth(), bitmap->GetHeight());

		bitmap->LockBits(
			&rect,
			ImageLockModeRead,
			PixelFormat16bppRGB565,
			bitmapData);

		U16 * texture = new U16[rect.Width * rect.Height];
		U16 * pixels = (U16*) bitmapData->Scan0;

		for(UINT row = 0; row < rect.Height; ++row)
		{
			UINT sourceRow = rect.Height - row - 1;
			for(UINT col = 0; col < rect.Width; ++col)
			{
				texture[row * rect.Width + col] = pixels[sourceRow * bitmapData->Stride / 2 + col];
			}
		}

		bitmap->UnlockBits(bitmapData);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, bitmap->GetWidth(), bitmap->GetHeight(), 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, texture);

		// create mipmaps
		if (bitmap->GetWidth() != 1 && bitmap->GetHeight() != 1) {

			int width = bitmap->GetWidth();
			int height= bitmap->GetHeight();
			int level = 0;

			// OK, for simplicity, assume square bitmap

			while (width > 1 || height > 1) {
				width = width >> 1;
				height = height >> 1;
				++level;

				for(UINT row = 0; row < height; ++row)
				{
					for(UINT col = 0; col < width; ++col)
					{
						EGL::Color p00 = EGL::Color::From565(texture[2 * row * (width * 2) + 2 * col]);
						EGL::Color p01 = EGL::Color::From565(texture[2 * row * (width * 2) + 2 * col + 1]);
						EGL::Color p10 = EGL::Color::From565(texture[(2 * row + 1) * (width * 2) + 2 * col]);
						EGL::Color p11 = EGL::Color::From565(texture[(2 * row + 1) * (width * 2) + 2 * col + 1]);

						EGL::Color result =
							EGL::Color((p00.R() + p01.R() + p10.R() + p11.R()) / 4,
									   (p00.G() + p01.G() + p10.G() + p11.G()) / 4,
									   (p00.B() + p01.B() + p10.B() + p11.B()) / 4,
									   (p00.A() + p01.A() + p10.A() + p11.A()) / 4);

						texture[row * width + col] = result.ConvertTo565();
					}
				}

				glTexImage2D(GL_TEXTURE_2D, level, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, texture);

			}
		}

		delete[] texture;
		return true;
	}
}

#else
namespace {
	struct DIBINFO : public BITMAPINFO
	{
		RGBQUAD	 arColors[255];    // Color table info - adds an extra 255 entries to palette

		operator LPBITMAPINFO()          { return (LPBITMAPINFO) this; }
		operator LPBITMAPINFOHEADER()    { return &bmiHeader;          }
		RGBQUAD* ColorTable()            { return bmiColors;           }
	};

	bool LoadTexture(const WCHAR *filename) {
		//
		HBITMAP hSourceBitmap  = SHLoadImageFile(filename);

		if (!hSourceBitmap)
			return false;

		BITMAP bm;
		GetObject(hSourceBitmap , sizeof(BITMAP), &bm);

		HBITMAP hTargetBitmap;
		void *pBuffer;

		//3.1 Initilize DIBINFO structure
		DIBINFO  dibInfo;
		dibInfo.bmiHeader.biBitCount = 24;
		dibInfo.bmiHeader.biClrImportant = 0;
		dibInfo.bmiHeader.biClrUsed = 0;
		dibInfo.bmiHeader.biCompression = 0;
		dibInfo.bmiHeader.biHeight = bm.bmHeight;
		dibInfo.bmiHeader.biPlanes = 1;
		dibInfo.bmiHeader.biSize = 40;
		dibInfo.bmiHeader.biSizeImage = bm.bmWidth*bm.bmHeight*3;
		dibInfo.bmiHeader.biWidth = bm.bmWidth;
		dibInfo.bmiHeader.biXPelsPerMeter = 3780;
		dibInfo.bmiHeader.biYPelsPerMeter = 3780;
		dibInfo.bmiColors[0].rgbBlue = 0;
		dibInfo.bmiColors[0].rgbGreen = 0;
		dibInfo.bmiColors[0].rgbRed = 0;
		dibInfo.bmiColors[0].rgbReserved = 0;

		//3.2 Create bitmap and receive pointer to points into pBuffer
		HDC hDC = ::GetDC(NULL);
		ASSERT(hDC);
		hTargetBitmap = CreateDIBSection(
			hDC,
			(const BITMAPINFO*)dibInfo,
			DIB_RGB_COLORS,
			(void**)&pBuffer,
			NULL,
			0);

		::ReleaseDC(NULL, hDC);


		//4. Copy source bitmap into the target bitmap.

		//4.1 Create 2 device contexts
		HDC memDC = CreateCompatibleDC(NULL);

		HDC targetDC = CreateCompatibleDC(NULL);

		//4.2 Select source bitmap into one DC, target into another
		HBITMAP hOldBitmap1 = (HBITMAP) SelectObject(memDC, hSourceBitmap);
		HBITMAP hOldBitmap2 = (HBITMAP) SelectObject(targetDC, hTargetBitmap);

		//4.3 Copy source bitmap into the target one
		BitBlt(targetDC, 0, 0, bm.bmWidth, bm.bmHeight, memDC, 0, 0, SRCCOPY);

		//4.4 Restore device contexts
		SelectObject(memDC, hOldBitmap1);
		SelectObject(targetDC, hOldBitmap2);
		DeleteDC(memDC);
		DeleteDC(targetDC);

#ifdef GL_GENERATE_MIPMAP
		glTexParameterx(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
#endif

#if 1
		//Here we can bitmap bits: pBuffer. Note:
		// 1. pBuffer contains 3 bytes per point
		// 2. Lines ane from the bottom to the top!
		// 3. Points in the line are from the left to the right
		// 4. Bytes in one point are BGR (blue, green, red) not RGB
		// 5. Don't delete pBuffer, it will be automatically deleted
		//    when delete hTargetBitmap
		U16 * transformedMap = new U16[bm.bmWidth * bm.bmHeight];
		U8 * bytes = reinterpret_cast<U8 *> (pBuffer);

		for (int x = 0; x < bm.bmWidth; ++x) {
			for (int y = 0; y < bm.bmHeight; ++y) {
				U8 * pByte = bytes + (x + y * bm.bmWidth) * 3;
				U8 r = pByte[0];
				U8 g = pByte[1];
				U8 b = pByte[2];

				U16 pixel = r >> 3 | (g & 0xFC) << 3 | (b & 0xF8) << 8;
				transformedMap[x + bm.bmWidth * y] = pixel;
			}
		}
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, bm.bmWidth, bm.bmHeight, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, transformedMap);
		delete[] transformedMap;
#else
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, bm.bmWidth, bm.bmHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
#endif

		DeleteObject(hSourceBitmap);
		DeleteObject(hTargetBitmap);

		return true;
	}
}
#endif


// OpenGL variables
EGLDisplay			g_display;				// EGL display
EGLSurface			g_surface;				// EGL rendering surface
EGLContext			g_context;				// EGL rendering context


// --------- BEGIN OF INCLUDED SNIPPET

struct sample_MATERIAL{
 GLfloat ambient[3];
 GLfloat diffuse[3];
 GLfloat specular[3];
 GLfloat emission[3];
 GLfloat alpha;
 GLfloat phExp;
 int   texture;
};
struct sample_TEXTURE{
 char * name;
 GLint  id;
};

static sample_MATERIAL materials [1] = {
 {{1.0f,1.0f,1.0f},	{1.0f,1.0f,1.0f},	{1.0f,1.0f,1.0f},	{1.0f,1.0f,1.0f},	1.0f,11.3137f,0} //Material #1
};

static sample_TEXTURE texture_maps [1] = {
{"dodge.bmp_0.bmp",0}
};

// 376 Verticies
// 822 Texture Coordinates
// 516 Normals
// 732 Triangles

static short face_indicies[732][9] = {
// Object #-1
	{29,30,27 ,0,1,2 ,0,1,2 }, {28,29,27 ,3,0,2 ,3,0,2 }, {31,32,30 ,4,5,1 ,4,5,1 },
	{21,34,35 ,6,7,8 ,6,7,8 }, {35,4,21 ,8,9,6 ,8,9,6 }, {34,26,25 ,7,10,11 ,7,10,11 },
	{25,35,34 ,11,8,7 ,11,8,7 }, {4,35,36 ,9,8,12 ,9,8,12 }, {36,17,4 ,12,13,9 ,12,13,9 },
	{16,17,36 ,14,13,12 ,14,13,12 }, {37,16,36 ,15,14,12 ,15,14,12 },
	{18,31,30 ,16,4,1 ,16,4,1 }, {1,18,30 ,17,16,1 ,17,16,1 }, {21,3,38 ,6,18,19 ,6,18,19 },
	{38,34,21 ,19,7,6 ,19,7,6 }, {29,38,3 ,0,19,18 ,0,19,18 }, {3,2,29 ,18,20,0 ,18,20,0 },
	{29,2,1 ,0,20,17 ,0,20,17 }, {29,1,30 ,0,17,1 ,0,17,1 }, {24,36,35 ,21,12,8 ,21,12,8 },
	{35,25,24 ,8,11,21 ,8,11,21 }, {39,37,36 ,22,15,12 ,22,15,12 },
	{24,39,36 ,21,22,12 ,21,22,12 }, {34,38,33 ,7,19,23 ,7,19,23 },
	{33,26,34 ,23,10,7 ,23,10,7 }, {38,29,28 ,19,0,3 ,19,0,3 }, {28,33,38 ,3,23,19 ,3,23,19 },
	{27,30,32 ,2,1,5 ,2,1,5 }, {11,7,23 ,24,25,26 ,24,25,26 }, {8,11,23 ,27,24,26 ,27,24,26 },
	{0,22,8 ,28,4,27 ,28,29,27 }, {9,20,21 ,29,30,6 ,30,31,6 }, {21,4,9 ,6,9,29 ,6,9,30 },
	{6,19,20 ,31,32,30 ,32,33,31 }, {20,9,6 ,30,29,31 ,31,30,32 },
	{15,9,4 ,33,29,9 ,34,30,9 }, {4,17,15 ,9,13,33 ,9,13,34 }, {16,14,15 ,14,34,33 ,14,35,34 },
	{17,16,15 ,13,14,33 ,13,14,34 }, {18,1,8 ,16,17,27 ,16,17,27 },
	{22,18,8 ,4,16,27 ,29,16,27 }, {10,3,21 ,35,18,6 ,36,18,6 },
	{21,20,10 ,6,30,35 ,6,31,36 }, {3,10,11 ,18,35,24 ,18,36,24 },
	{11,2,3 ,24,20,18 ,24,20,18 }, {11,8,1 ,24,27,17 ,24,27,17 },
	{1,2,11 ,17,20,24 ,17,20,24 }, {9,15,13 ,29,33,36 ,30,34,37 },
	{13,6,9 ,36,31,29 ,37,32,30 }, {12,13,15 ,37,36,33 ,38,37,34 },
	{14,12,15 ,34,37,33 ,35,38,34 }, {5,10,20 ,38,35,30 ,39,36,31 },
	{20,19,5 ,30,32,38 ,31,33,39 }, {7,11,10 ,25,24,35 ,25,24,36 },
	{10,5,7 ,35,38,25 ,36,39,25 }, {8,23,0 ,27,26,28 ,27,26,28 },
	{53,42,49 ,39,40,41 ,40,41,42 }, {49,52,53 ,41,42,39 ,42,43,40 },
	{46,54,52 ,43,44,42 ,44,45,43 }, {52,49,46 ,42,41,43 ,43,42,44 },
	{43,55,54 ,45,46,44 ,46,47,45 }, {54,46,43 ,44,43,45 ,45,44,46 },
	{53,52,57 ,39,42,47 ,40,43,48 }, {56,53,57 ,48,39,47 ,49,40,48 },
	{57,52,54 ,47,42,44 ,48,43,45 }, {58,57,54 ,49,47,44 ,50,48,45 },
	{58,54,55 ,49,44,46 ,50,45,47 }, {59,58,55 ,50,49,46 ,51,50,47 },
	{49,42,40 ,41,40,51 ,42,41,52 }, {40,50,49 ,51,52,41 ,52,53,42 },
	{50,47,46 ,52,53,43 ,53,54,44 }, {46,49,50 ,43,41,52 ,44,42,53 },
	{47,44,43 ,53,54,45 ,54,55,46 }, {43,46,47 ,45,43,53 ,46,44,54 },
	{40,41,51 ,51,55,56 ,52,56,57 }, {50,40,51 ,52,51,56 ,53,52,57 },
	{51,48,47 ,56,57,53 ,57,58,54 }, {50,51,47 ,52,56,53 ,53,57,54 },
	{48,45,44 ,57,58,54 ,58,59,55 }, {47,48,44 ,53,57,54 ,54,58,55 },
	{130,131,129 ,59,60,61 ,60,61,62 }, {133,134,132 ,62,63,64 ,63,64,65 },
	{129,131,134 ,61,60,63 ,62,61,64 }, {135,130,129 ,65,59,61 ,66,60,62 },
	{136,135,129 ,66,65,61 ,67,66,62 }, {133,132,135 ,62,64,65 ,63,65,66 },
	{135,136,133 ,65,66,62 ,66,67,63 }, {134,131,132 ,63,60,64 ,64,61,65 },
	{138,139,137 ,67,68,69 ,68,69,70 }, {141,142,140 ,70,71,72 ,71,72,73 },
	{141,140,143 ,70,72,73 ,71,73,74 }, {135,145,144 ,65,74,75 ,66,75,76 },
	{135,132,145 ,65,64,74 ,66,65,75 }, {132,131,146 ,64,60,76 ,65,61,77 },
	{146,147,132 ,76,77,64 ,77,78,65 }, {130,148,131 ,59,78,60 ,60,79,61 },
	{150,151,149 ,79,80,81 ,80,81,82 }, {154,151,152 ,82,80,83 ,83,81,84 },
	{153,154,152 ,84,82,83 ,85,83,84 }, {153,152,155 ,84,83,85 ,85,84,86 },
	{156,150,149 ,86,79,81 ,87,80,82 }, {157,151,154 ,87,80,82 ,88,81,83 },
	{151,150,158 ,80,79,88 ,81,80,89 }, {152,151,158 ,83,80,88 ,84,81,89 },
	{150,156,130 ,79,86,59 ,80,87,60 }, {130,158,150 ,59,88,79 ,60,89,80 },
	{151,157,149 ,80,87,81 ,81,88,82 }, {155,152,158 ,85,83,88 ,86,84,89 },
	{159,155,158 ,89,85,88 ,90,86,89 }, {159,158,130 ,89,88,59 ,90,89,60 },
	{135,159,130 ,65,89,59 ,66,90,60 }, {25,160,24 ,90,91,90 ,91,92,93 },
	{25,26,161 ,92,92,93 ,94,95,96 }, {140,142,161 ,72,71,93 ,73,72,96 },
	{163,27,162 ,94,95,96 ,97,98,99 }, {165,166,164 ,97,98,99 ,100,101,102 },
	{164,163,167 ,99,94,100 ,102,97,103 }, {166,28,163 ,98,101,94 ,101,104,97 },
	{166,165,168 ,98,97,102 ,101,100,105 }, {161,26,168 ,93,103,102 ,96,106,105 },
	{166,163,164 ,98,94,99 ,101,97,102 }, {163,162,167 ,94,96,100 ,97,99,103 },
	{28,27,163 ,104,104,94 ,107,98,97 }, {28,166,33 ,105,98,105 ,108,101,109 },
	{170,25,169 ,106,107,108 ,110,111,112 }, {156,148,130 ,86,78,59 ,87,79,60 },
	{142,148,156 ,71,78,86 ,72,79,87 }, {156,170,142 ,86,106,71 ,87,110,72 },
	{170,169,142 ,106,108,71 ,110,112,72 }, {166,168,33 ,98,102,109 ,101,105,109 },
	{144,172,171 ,75,110,111 ,76,113,114 }, {169,25,161 ,108,112,93 ,112,115,96 },
	{147,173,145 ,77,113,74 ,78,116,75 }, {145,132,147 ,74,64,77 ,75,65,78 },
	{147,146,174 ,77,76,114 ,78,77,117 }, {174,175,147 ,114,115,77 ,117,118,78 },
	{172,173,139 ,110,113,68 ,113,116,69 }, {139,176,172 ,68,116,110 ,69,119,113 },
	{173,147,175 ,113,77,115 ,116,78,118 }, {139,173,175 ,68,113,115 ,69,116,118 },
	{175,174,177 ,115,114,117 ,118,117,120 }, {177,178,175 ,117,118,115 ,120,121,118 },
	{139,175,178 ,68,115,118 ,69,118,121 }, {137,139,178 ,69,68,118 ,70,69,121 },
	{181,138,137 ,119,67,69 ,122,68,70 }, {179,181,137 ,120,119,69 ,123,122,70 },
	{137,178,179 ,69,118,120 ,70,121,123 }, {180,179,178 ,121,120,118 ,124,123,121 },
	{177,180,178 ,117,121,118 ,120,124,121 }, {177,182,183 ,117,122,123 ,120,125,126 },
	{183,180,177 ,123,121,117 ,126,124,120 }, {172,186,171 ,110,124,111 ,113,127,114 },
	{173,172,144 ,113,110,75 ,116,113,76 }, {145,173,144 ,74,113,75 ,75,116,76 },
	{172,176,186 ,110,116,124 ,113,119,127 }, {171,185,184 ,111,125,126 ,114,128,129 },
	{184,144,171 ,126,75,111 ,129,76,114 }, {144,184,135 ,75,126,65 ,76,129,66 },
	{188,186,176 ,127,124,116 ,130,127,119 }, {138,188,176 ,67,127,116 ,68,130,119 },
	{138,176,139 ,67,116,68 ,68,119,69 }, {187,188,138 ,128,127,67 ,131,130,68 },
	{138,181,187 ,67,119,128 ,68,122,131 }, {148,141,146 ,78,70,76 ,79,71,77 },
	{131,148,146 ,60,78,76 ,61,79,77 }, {148,142,141 ,78,71,70 ,79,72,71 },
	{160,25,170 ,91,129,106 ,92,132,110 }, {143,140,161 ,73,72,93 ,74,73,96 },
	{168,143,161 ,102,73,93 ,105,74,96 }, {191,190,57 ,130,131,132 ,133,134,135 },
	{58,191,57 ,133,130,133 ,136,133,137 }, {57,190,189 ,134,131,135 ,138,134,139 },
	{56,57,189 ,136,136,135 ,140,135,139 }, {191,58,59 ,130,137,137 ,133,141,142 },
	{59,193,191 ,138,139,130 ,143,144,133 }, {160,39,24 ,91,140,140 ,92,145,146 },
	{193,59,195 ,139,141,142 ,144,147,148 }, {192,193,195 ,143,139,142 ,149,144,148 },
	{192,195,196 ,143,142,144 ,149,148,150 }, {194,192,196 ,145,143,144 ,151,149,150 },
	{185,171,194 ,125,111,145 ,128,114,151 }, {194,196,185 ,145,144,125 ,151,150,128 },
	{170,156,149 ,106,86,81 ,110,87,82 }, {168,165,143 ,102,97,73 ,105,100,74 },
	{197,174,146 ,146,114,76 ,152,117,77 }, {141,197,146 ,70,146,76 ,71,152,77 },
	{142,169,161 ,71,108,93 ,72,112,96 }, {174,197,182 ,114,146,122 ,117,152,125 },
	{182,177,174 ,122,117,114 ,125,120,117 }, {162,27,32 ,96,147,147 ,99,153,154 },
	{168,26,33 ,102,148,148 ,105,155,156 }, {170,149,160 ,106,81,91 ,110,82,92 },
	{120,76,126 ,149,150,151 ,157,158,159 }, {127,128,121 ,152,153,154 ,160,161,162 },
	{120,126,127 ,149,151,152 ,157,159,160 }, {76,81,126 ,150,155,151 ,158,163,159 },
	{81,125,126 ,155,156,151 ,163,164,159 }, {81,121,128 ,155,154,153 ,163,162,161 },
	{128,125,81 ,153,156,155 ,161,164,163 }, {120,127,121 ,149,152,154 ,157,160,162 },
	{85,104,124 ,157,158,159 ,165,166,167 }, {116,75,123 ,160,161,162 ,168,169,170 },
	{123,75,70 ,162,161,163 ,170,169,171 }, {122,81,100 ,164,155,165 ,172,163,173 },
	{121,81,122 ,154,155,164 ,162,163,172 }, {111,120,121 ,166,149,154 ,174,157,162 },
	{121,110,111 ,154,167,166 ,162,175,174 }, {115,76,120 ,168,150,149 ,176,158,157 },
	{113,91,78 ,169,170,171 ,177,178,179 }, {80,83,112 ,172,173,174 ,180,181,182 },
	{113,80,112 ,169,172,174 ,177,180,182 }, {112,83,82 ,174,173,175 ,182,181,183 },
	{91,92,78 ,170,176,171 ,178,184,179 }, {113,88,80 ,169,177,172 ,177,185,180 },
	{113,112,90 ,169,174,178 ,177,182,186 }, {91,113,90 ,170,169,178 ,178,177,186 },
	{76,92,91 ,150,176,170 ,158,184,178 }, {91,90,76 ,170,178,150 ,178,186,158 },
	{88,113,78 ,177,169,171 ,185,177,179 }, {82,89,90 ,175,179,178 ,183,187,186 },
	{112,82,90 ,174,175,178 ,182,183,186 }, {89,81,76 ,179,155,150 ,187,163,158 },
	{90,89,76 ,178,179,150 ,186,187,158 }, {94,6,13 ,180,181,181 ,188,189,190 },
	{19,6,77 ,182,182,183 ,191,192,193 }, {116,123,77 ,160,162,183 ,168,170,193 },
	{23,118,60 ,184,185,186 ,194,97,195 }, {86,69,119 ,187,188,189 ,196,197,198 },
	{118,119,68 ,185,189,190 ,97,198,199 }, {7,86,118 ,191,187,185 ,200,196,97 },
	{69,86,114 ,188,187,192 ,197,196,201 }, {19,77,114 ,193,183,192 ,202,193,201 },
	{118,86,119 ,185,187,189 ,97,196,198 }, {60,118,68 ,186,185,190 ,195,97,199 },
	{23,7,118 ,194,194,185 ,194,203,97 }, {86,7,5 ,187,195,195 ,196,204,205 },
	{6,98,117 ,196,197,198 ,206,207,208 }, {115,92,76 ,168,176,150 ,176,184,158 },
	{92,115,116 ,176,168,160 ,184,176,168 }, {116,98,92 ,160,197,176 ,168,207,184 },
	{117,98,116 ,198,197,160 ,208,207,168 }, {114,86,5 ,192,187,199 ,201,196,205 },
	{101,100,74 ,200,165,201 ,209,173,210 }, {6,117,77 ,202,198,183 ,211,208,193 },
	{122,109,110 ,164,203,167 ,172,212,175 }, {110,121,122 ,167,154,164 ,175,162,172 },
	{87,111,110 ,204,166,167 ,213,174,175 }, {110,107,87 ,167,205,204 ,175,214,213 },
	{85,109,101 ,157,203,200 ,165,212,209 }, {101,102,85 ,200,206,157 ,209,215,165 },
	{109,85,107 ,203,157,205 ,212,165,214 }, {110,109,107 ,167,203,205 ,175,212,214 },
	{105,87,107 ,207,204,205 ,216,213,214 }, {107,106,105 ,205,208,207 ,214,217,216 },
	{85,124,106 ,157,159,208 ,165,167,217 }, {107,85,106 ,205,157,208 ,214,165,217 },
	{124,104,103 ,159,158,209 ,167,166,218 }, {108,106,124 ,210,208,159 ,219,217,167 },
	{103,108,124 ,209,210,159 ,218,219,167 }, {62,105,106 ,211,207,208 ,220,216,217 },
	{108,62,106 ,210,211,208 ,219,220,217 }, {73,72,105 ,212,213,207 ,221,222,216 },
	{105,62,73 ,207,211,212 ,216,220,221 }, {64,101,74 ,214,200,201 ,223,209,210 },
	{109,122,100 ,203,164,165 ,212,172,173 }, {101,109,100 ,200,203,165 ,209,212,173 },
	{102,101,64 ,206,200,214 ,215,209,223 }, {99,84,74 ,215,216,201 ,224,225,210 },
	{74,100,99 ,201,165,215 ,210,173,224 }, {99,100,81 ,215,165,155 ,224,173,163 },
	{63,104,102 ,217,158,206 ,226,166,215 }, {64,63,102 ,214,217,206 ,223,226,215 },
	{102,104,85 ,206,158,157 ,215,166,165 }, {104,63,61 ,158,217,218 ,166,226,227 },
	{61,103,104 ,218,209,158 ,227,218,166 }, {115,120,111 ,168,149,166 ,176,157,174 },
	{75,115,111 ,161,168,166 ,169,176,174 }, {116,115,75 ,160,168,161 ,168,176,169 },
	{6,94,98 ,219,180,197 ,228,188,207 }, {123,70,77 ,162,163,183 ,170,171,193 },
	{70,114,77 ,163,192,183 ,171,201,193 }, {95,48,51 ,220,221,221 ,229,230,231 },
	{97,95,51 ,222,220,223 ,232,229,231 }, {51,41,67 ,224,224,225 ,233,234,235 },
	{97,51,67 ,222,226,225 ,232,236,235 }, {45,48,95 ,227,227,220 ,237,238,229 },
	{95,66,45 ,220,228,229 ,229,239,240 }, {12,94,13 ,230,180,230 ,241,188,242 },
	{66,96,93 ,228,231,232 ,239,243,244 }, {45,66,93 ,233,228,232 ,245,239,244 },
	{96,65,79 ,231,234,235 ,243,246,247 }, {93,96,79 ,232,231,235 ,244,243,247 },
	{65,74,84 ,234,201,216 ,246,210,225 }, {84,79,65 ,216,235,234 ,225,247,246 },
	{92,98,78 ,176,197,171 ,184,207,179 }, {69,114,70 ,188,192,163 ,197,201,171 },
	{71,75,111 ,236,161,166 ,248,169,174 }, {87,71,111 ,204,236,166 ,213,248,174 },
	{117,116,77 ,198,160,183 ,208,168,193 }, {72,71,87 ,213,236,204 ,222,248,213 },
	{87,105,72 ,204,207,213 ,213,216,222 }, {23,60,0 ,237,186,237 ,249,195,250 },
	{19,114,5 ,238,192,238 ,251,201,252 }, {78,98,94 ,171,197,180 ,179,207,188 },
	{201,221,220 ,239,240,241 ,253,254,255 }, {219,201,220 ,242,239,241 ,256,253,255 },
	{220,222,200 ,241,243,244 ,255,257,258 }, {219,220,200 ,242,241,244 ,256,255,258 },
	{222,220,184 ,243,241,245 ,257,255,259 }, {184,185,222 ,246,246,243 ,260,261,257 },
	{221,135,184 ,240,247,247 ,254,262,259 }, {184,220,221 ,248,241,240 ,263,255,254 },
	{37,223,216 ,249,250,251 ,264,265,266 }, {16,37,216 ,252,252,251 ,267,268,266 },
	{223,224,204 ,250,253,254 ,265,269,270 }, {216,223,204 ,251,250,254 ,266,265,270 },
	{226,198,206 ,255,256,257 ,271,272,273 }, {225,226,206 ,258,255,257 ,274,271,273 },
	{228,227,203 ,259,260,261 ,275,276,277 }, {202,228,203 ,262,259,261 ,278,275,277 },
	{229,199,200 ,263,264,244 ,279,280,258 }, {200,222,229 ,244,243,263 ,258,257,279 },
	{198,226,228 ,256,255,259 ,272,271,275 }, {228,202,198 ,259,262,256 ,275,278,272 },
	{55,43,209 ,265,265,266 ,281,282,283 }, {209,230,55 ,266,267,268 ,283,284,285 },
	{230,209,199 ,267,266,264 ,284,283,280 }, {199,229,230 ,264,263,267 ,280,279,284 },
	{204,224,225 ,254,253,258 ,270,269,274 }, {225,206,204 ,258,257,254 ,274,273,270 },
	{223,37,39 ,250,269,269 ,265,286,287 }, {160,223,39 ,270,250,270 ,288,265,287 },
	{160,149,224 ,271,271,253 ,289,290,269 }, {223,160,224 ,250,272,253 ,265,291,269 },
	{227,228,153 ,260,259,273 ,276,275,292 }, {153,155,227 ,274,274,260 ,293,294,276 },
	{226,154,153 ,255,275,275 ,271,295,292 }, {153,228,226 ,276,259,255 ,296,275,271 },
	{230,195,59 ,267,277,277 ,284,297,298 }, {55,230,59 ,278,267,278 ,299,284,300 },
	{195,230,229 ,279,267,263 ,301,284,279 }, {229,196,195 ,263,280,280 ,279,302,303 },
	{154,226,225 ,281,255,258 ,304,271,274 }, {157,154,225 ,282,282,258 ,305,306,274 },
	{224,149,157 ,253,283,283 ,269,307,308 }, {157,225,224 ,284,258,253 ,309,274,269 },
	{196,229,222 ,285,263,243 ,310,279,257 }, {185,196,222 ,286,286,243 ,311,312,257 },
	{201,219,218 ,239,242,287 ,253,256,313 }, {213,201,218 ,288,239,287 ,314,253,313 },
	{218,219,200 ,287,242,244 ,313,256,258 }, {215,218,200 ,289,287,244 ,315,313,258 },
	{99,218,215 ,290,287,289 ,316,313,315 }, {215,84,99 ,289,291,291 ,315,317,318 },
	{99,81,213 ,292,292,288 ,319,320,314 }, {213,218,99 ,288,287,293 ,314,313,321 },
	{14,16,216 ,294,294,251 ,322,267,266 }, {217,14,216 ,295,296,251 ,323,324,266 },
	{217,216,204 ,295,251,254 ,323,266,270 }, {207,217,204 ,297,295,254 ,325,323,270 },
	{211,208,206 ,298,299,257 ,326,327,273 }, {198,211,206 ,256,298,257 ,272,326,273 },
	{214,202,203 ,300,262,261 ,328,278,277 }, {212,214,203 ,301,300,261 ,329,328,277 },
	{200,199,205 ,244,264,302 ,258,280,330 }, {205,215,200 ,302,289,244 ,330,315,258 },
	{214,211,198 ,300,298,256 ,328,326,272 }, {198,202,214 ,256,262,300 ,272,278,328 },
	{209,43,44 ,266,303,303 ,283,331,332 }, {44,210,209 ,304,305,266 ,333,334,283 },
	{199,209,210 ,264,266,305 ,280,283,334 }, {210,205,199 ,305,302,264 ,334,330,280 },
	{208,207,204 ,299,297,254 ,327,325,270 }, {204,206,208 ,254,257,299 ,270,273,327 },
	{217,94,12 ,295,306,306 ,323,335,336 }, {14,217,12 ,307,295,307 ,337,323,338 },
	{94,217,207 ,308,295,297 ,339,323,325 }, {78,94,207 ,309,309,297 ,340,341,325 },
	{83,214,212 ,310,300,301 ,342,328,329 }, {212,82,83 ,301,311,311 ,329,343,344 },
	{83,80,211 ,312,312,298 ,345,346,326 }, {211,214,83 ,298,300,313 ,326,328,347 },
	{210,44,45 ,305,314,314 ,334,348,349 }, {93,210,45 ,315,305,315 ,350,334,349 },
	{205,210,93 ,302,305,316 ,330,334,351 }, {93,79,205 ,317,317,302 ,352,353,330 },
	{80,88,208 ,318,318,299 ,354,355,327 }, {211,80,208 ,298,319,299 ,326,354,327 },
	{88,78,207 ,320,320,297 ,356,357,325 }, {207,208,88 ,297,299,321 ,325,327,358 },
	{79,84,215 ,322,322,289 ,359,360,315 }, {205,79,215 ,302,323,289 ,330,361,315 },
	{250,241,231 ,324,324,324 ,362,363,364 }, {231,240,250 ,324,324,324 ,364,365,362 },
	{249,250,240 ,325,325,325 ,366,362,365 }, {240,239,249 ,325,325,325 ,365,367,366 },
	{248,249,239 ,326,326,326 ,368,366,367 }, {239,238,248 ,326,326,326 ,367,369,368 },
	{247,248,238 ,327,327,327 ,368,368,369 }, {238,237,247 ,327,327,327 ,369,370,368 },
	{246,247,237 ,328,328,328 ,366,368,370 }, {237,236,246 ,328,328,328 ,370,371,366 },
	{245,246,236 ,329,329,329 ,372,366,371 }, {236,235,245 ,329,329,329 ,371,373,372 },
	{244,245,235 ,330,330,330 ,374,372,373 }, {235,234,244 ,330,330,330 ,373,375,374 },
	{243,244,234 ,331,331,331 ,376,374,375 }, {234,233,243 ,331,331,331 ,375,377,376 },
	{242,243,233 ,332,332,332 ,376,376,377 }, {233,232,242 ,332,332,332 ,377,377,376 },
	{241,242,232 ,333,333,333 ,363,376,377 }, {232,231,241 ,333,333,333 ,377,364,363 },
	{255,267,266 ,324,324,324 ,378,379,380 }, {266,256,255 ,324,324,324 ,380,381,378 },
	{256,266,265 ,325,325,325 ,381,380,382 }, {265,257,256 ,325,325,325 ,382,383,381 },
	{257,265,264 ,326,326,326 ,383,382,384 }, {264,258,257 ,326,326,326 ,384,385,383 },
	{258,264,263 ,327,327,327 ,385,384,384 }, {263,259,258 ,327,327,327 ,384,386,385 },
	{259,263,262 ,328,328,328 ,386,384,382 }, {262,260,259 ,328,328,328 ,382,387,386 },
	{260,262,261 ,329,329,329 ,387,382,380 }, {261,251,260 ,329,329,329 ,380,388,387 },
	{251,261,270 ,330,330,330 ,388,380,389 }, {270,252,251 ,330,330,330 ,389,390,388 },
	{252,270,269 ,331,331,331 ,390,389,391 }, {269,253,252 ,331,331,331 ,391,392,390 },
	{253,269,268 ,332,332,332 ,392,391,391 }, {268,254,253 ,332,332,332 ,391,393,392 },
	{254,268,267 ,333,333,333 ,393,391,379 }, {267,255,254 ,333,333,333 ,379,378,393 },
	{290,281,271 ,334,334,334 ,394,395,396 }, {271,280,290 ,334,334,334 ,396,397,394 },
	{289,290,280 ,325,325,325 ,398,394,397 }, {280,279,289 ,325,325,325 ,397,399,398 },
	{288,289,279 ,335,335,335 ,400,398,399 }, {279,278,288 ,335,335,335 ,399,401,400 },
	{287,288,278 ,336,336,336 ,400,400,401 }, {278,277,287 ,336,336,336 ,401,402,400 },
	{286,287,277 ,337,337,337 ,398,400,402 }, {277,276,286 ,337,337,337 ,402,403,398 },
	{285,286,276 ,329,329,329 ,404,398,403 }, {276,275,285 ,329,329,329 ,403,405,404 },
	{284,285,275 ,338,338,338 ,395,404,405 }, {275,274,284 ,338,338,338 ,405,396,395 },
	{283,284,274 ,339,339,339 ,406,395,396 }, {274,273,283 ,339,339,339 ,396,407,406 },
	{282,283,273 ,332,332,332 ,406,406,407 }, {273,272,282 ,332,332,332 ,407,407,406 },
	{281,282,272 ,340,340,340 ,395,406,407 }, {272,271,281 ,340,340,340 ,407,396,395 },
	{295,307,306 ,334,334,334 ,408,409,410 }, {306,296,295 ,334,334,334 ,410,411,408 },
	{296,306,305 ,325,325,325 ,411,410,412 }, {305,297,296 ,325,325,325 ,412,413,411 },
	{297,305,304 ,335,335,335 ,413,412,414 }, {304,298,297 ,335,335,335 ,414,415,413 },
	{298,304,303 ,336,336,336 ,415,414,414 }, {303,299,298 ,336,336,336 ,414,415,415 },
	{299,303,302 ,337,337,337 ,415,414,412 }, {302,300,299 ,337,337,337 ,412,416,415 },
	{300,302,301 ,329,329,329 ,416,412,410 }, {301,291,300 ,329,329,329 ,410,411,416 },
	{291,301,310 ,338,338,338 ,411,410,417 }, {310,292,291 ,338,338,338 ,417,408,411 },
	{292,310,309 ,339,339,339 ,408,417,418 }, {309,293,292 ,339,339,339 ,418,419,408 },
	{293,309,308 ,332,332,332 ,419,418,418 }, {308,294,293 ,332,332,332 ,418,419,419 },
	{294,308,307 ,340,340,340 ,419,418,409 }, {307,295,294 ,340,340,340 ,409,408,419 },
	{18,22,312 ,341,341,342 ,420,421,422 }, {18,312,314 ,341,342,343 ,423,422,424 },
	{314,31,18 ,343,341,341 ,424,425,426 }, {312,60,311 ,342,344,345 ,422,427,428 },
	{311,313,314 ,345,346,343 ,428,429,424 }, {314,312,311 ,343,342,345 ,424,422,428 },
	{32,31,314 ,341,341,343 ,430,431,424 }, {162,32,314 ,341,341,343 ,432,433,424 },
	{313,167,162 ,346,347,347 ,429,434,435 }, {0,60,312 ,341,341,342 ,436,437,422 },
	{22,0,312 ,341,341,342 ,438,439,422 }, {162,314,313 ,344,343,346 ,440,424,429 },
	{68,311,60 ,348,345,348 ,441,428,442 }, {315,73,62 ,349,350,350 ,443,444,445 },
	{180,183,332 ,350,350,351 ,446,447,448 }, {325,67,41 ,352,353,353 ,449,450,451 },
	{56,189,340 ,353,353,354 ,452,453,454 }, {103,61,322 ,355,355,356 ,455,456,457 },
	{339,187,181 ,357,355,355 ,458,459,460 }, {40,42,53 ,358,358,358 ,461,462,463 },
	{53,56,41 ,359,359,359 ,464,465,466 }, {41,40,53 ,359,359,359 ,467,468,464 },
	{56,340,325 ,353,354,352 ,465,454,449 }, {325,41,56 ,352,353,353 ,449,451,469 },
	{340,339,322 ,354,357,356 ,454,458,457 }, {322,325,340 ,356,352,354 ,457,449,454 },
	{339,181,103 ,357,355,355 ,458,470,471 }, {103,322,339 ,355,356,357 ,471,457,458 },
	{181,179,108 ,360,360,360 ,472,473,474 }, {108,103,181 ,360,360,360 ,475,471,472 },
	{179,180,62 ,361,361,361 ,476,477,478 }, {62,108,179 ,361,361,361 ,479,474,476 },
	{180,332,315 ,350,351,349 ,477,448,443 }, {315,62,180 ,349,350,350 ,443,445,480 },
	{313,332,333 ,362,351,362 ,481,448,482 }, {313,333,334 ,363,363,364 ,481,482,483 },
	{313,334,335 ,365,364,366 ,481,483,484 }, {335,336,337 ,366,367,367 ,484,485,486 },
	{338,313,335 ,368,369,366 ,487,481,484 }, {335,337,338 ,366,367,368 ,484,486,487 },
	{167,313,338 ,370,370,371 ,488,489,487 }, {164,167,338 ,372,372,371 ,490,491,487 },
	{164,338,337 ,373,371,374 ,492,487,486 }, {164,337,165 ,373,374,373 ,493,486,494 },
	{348,339,340 ,375,375,375 ,495,458,454 }, {347,348,340 ,367,367,367 ,496,495,454 },
	{346,347,340 ,367,367,367 ,497,496,454 }, {345,346,340 ,367,367,367 ,498,497,454 },
	{343,344,345 ,367,367,367 ,499,500,498 }, {342,343,345 ,367,367,367 ,501,499,498 },
	{341,342,345 ,367,367,367 ,502,501,498 }, {340,341,345 ,367,367,367 ,454,502,498 },
	{189,190,341 ,376,376,377 ,503,504,502 }, {341,340,189 ,377,354,376 ,502,454,503 },
	{190,191,342 ,378,378,379 ,505,506,501 }, {341,190,342 ,377,378,379 ,502,507,501 },
	{193,192,344 ,380,380,381 ,508,509,500 }, {344,343,193 ,381,382,380 ,500,499,508 },
	{192,194,345 ,383,383,384 ,510,511,498 }, {344,192,345 ,381,383,384 ,500,512,498 },
	{191,193,343 ,385,385,382 ,513,514,499 }, {342,191,343 ,379,385,382 ,501,515,499 },
	{165,337,336 ,386,374,387 ,516,486,485 }, {336,143,165 ,387,386,386 ,485,517,518 },
	{335,141,143 ,388,389,389 ,484,519,520 }, {336,335,143 ,387,388,389 ,485,484,521 },
	{183,182,333 ,390,390,391 ,522,523,482 }, {333,332,183 ,391,390,390 ,482,448,524 },
	{197,334,333 ,392,393,391 ,525,483,482 }, {182,197,333 ,392,392,391 ,526,527,482 },
	{141,335,334 ,394,388,393 ,528,484,483 }, {197,141,334 ,394,394,393 ,527,529,483 },
	{187,339,348 ,395,357,396 ,530,458,495 }, {188,187,348 ,395,395,396 ,531,530,495 },
	{188,348,347 ,397,396,398 ,532,495,496 }, {186,188,347 ,399,399,398 ,533,534,496 },
	{171,186,347 ,400,400,398 ,535,536,496 }, {347,346,171 ,398,401,402 ,496,497,537 },
	{194,171,346 ,403,403,401 ,538,537,497 }, {346,345,194 ,401,384,403 ,497,498,539 },
	{319,315,311 ,404,349,404 ,540,443,541 }, {320,319,311 ,405,406,406 ,542,540,541 },
	{318,320,311 ,407,405,408 ,543,542,541 }, {316,317,318 ,409,409,407 ,544,545,543 },
	{331,316,318 ,410,409,407 ,546,544,543 }, {311,331,318 ,411,410,407 ,541,546,543 },
	{68,119,331 ,372,372,412 ,547,548,546 }, {311,68,331 ,413,413,412 ,541,547,546 },
	{119,69,316 ,373,373,374 ,549,550,544 }, {316,331,119 ,374,412,373 ,544,546,551 },
	{313,311,315 ,414,414,414 ,552,553,443 }, {315,332,313 ,414,414,414 ,443,448,554 },
	{325,322,321 ,415,415,415 ,449,457,555 }, {325,321,323 ,409,409,409 ,449,555,556 },
	{325,323,324 ,409,409,409 ,449,556,557 }, {325,324,327 ,409,409,409 ,449,557,558 },
	{327,329,326 ,409,409,409 ,558,559,560 }, {327,326,328 ,409,409,409 ,558,560,561 },
	{330,325,327 ,409,409,409 ,562,449,558 }, {327,328,330 ,409,409,409 ,558,561,562 },
	{330,97,67 ,377,376,376 ,562,563,564 }, {67,325,330 ,376,352,377 ,565,449,562 },
	{97,330,328 ,378,377,379 ,566,562,561 }, {95,97,328 ,378,378,379 ,567,568,561 },
	{329,96,66 ,381,380,380 ,559,569,570 }, {66,326,329 ,380,382,381 ,571,560,559 },
	{96,329,327 ,383,381,384 ,572,559,558 }, {65,96,327 ,383,383,384 ,573,574,558 },
	{95,328,326 ,385,379,382 ,575,561,560 }, {66,95,326 ,385,385,382 ,576,577,560 },
	{69,70,317 ,386,386,387 ,578,579,545 }, {317,316,69 ,387,374,386 ,545,544,578 },
	{318,317,70 ,388,387,389 ,543,545,580 }, {75,318,70 ,389,388,389 ,581,543,582 },
	{319,72,73 ,391,390,390 ,540,583,584 }, {73,315,319 ,390,390,391 ,585,443,540 },
	{71,72,319 ,392,392,391 ,586,587,540 }, {320,71,319 ,393,392,391 ,542,588,540 },
	{75,71,320 ,394,394,393 ,589,588,542 }, {318,75,320 ,388,394,393 ,543,590,542 },
	{61,63,321 ,395,395,396 ,591,592,555 }, {322,61,321 ,356,395,396 ,457,591,555 },
	{63,64,323 ,416,416,417 ,593,594,556 }, {321,63,323 ,396,397,417 ,555,593,556 },
	{323,64,74 ,417,418,418 ,556,595,596 }, {74,324,323 ,402,401,417 ,596,557,556 },
	{324,74,65 ,401,403,403 ,557,597,598 }, {65,327,324 ,403,384,401 ,599,558,557 },
	{260,251,252 ,367,367,367 ,600,601,602 }, {260,252,253 ,367,367,367 ,603,604,605 },
	{260,253,254 ,367,367,367 ,606,607,608 }, {260,254,255 ,367,367,367 ,606,609,610 },
	{260,255,256 ,367,367,367 ,611,612,613 }, {260,256,257 ,367,367,367 ,600,613,614 },
	{258,259,260 ,367,367,367 ,615,616,617 }, {257,258,260 ,367,367,367 ,614,618,603 },
	{270,261,262 ,419,419,419 ,619,620,621 }, {269,270,262 ,420,420,420 ,622,602,623 },
	{268,269,262 ,421,421,421 ,624,605,625 }, {267,268,262 ,422,422,422 ,626,627,628 },
	{266,267,262 ,423,423,423 ,629,630,631 }, {265,266,262 ,424,424,424 ,632,613,633 },
	{264,265,262 ,425,425,425 ,618,634,631 }, {262,263,264 ,426,426,426 ,633,635,636 },
	{234,235,236 ,409,409,409 ,637,638,606 }, {233,234,236 ,409,409,409 ,639,640,606 },
	{232,233,236 ,409,409,409 ,641,642,603 }, {231,232,236 ,409,409,409 ,643,608,600 },
	{240,231,236 ,409,409,409 ,644,645,617 }, {239,240,236 ,409,409,409 ,646,647,648 },
	{238,239,236 ,409,409,409 ,618,649,617 }, {236,237,238 ,409,409,409 ,648,635,650 },
	{246,245,244 ,427,427,427 ,651,652,653 }, {246,244,243 ,428,428,428 ,654,655,656 },
	{246,243,242 ,429,429,429 ,657,658,659 }, {246,242,241 ,430,430,430 ,657,660,661 },
	{246,241,250 ,431,431,431 ,654,662,663 }, {246,250,249 ,432,432,432 ,664,647,614 },
	{248,247,246 ,433,433,433 ,615,665,666 }, {249,248,246 ,434,434,434 ,614,650,667 },
	{300,291,292 ,367,367,367 ,668,669,670 }, {300,292,293 ,367,367,367 ,671,672,673 },
	{300,293,294 ,367,367,367 ,674,675,676 }, {300,294,295 ,367,367,367 ,674,677,678 },
	{300,295,296 ,367,367,367 ,671,679,680 }, {300,296,297 ,367,367,367 ,668,680,681 },
	{298,299,300 ,367,367,367 ,682,683,684 }, {297,298,300 ,367,367,367 ,681,685,671 },
	{310,301,302 ,435,435,435 ,686,687,674 }, {309,310,302 ,436,436,436 ,688,689,674 },
	{308,309,302 ,421,421,421 ,690,673,671 }, {307,308,302 ,437,437,437 ,691,676,668 },
	{306,307,302 ,438,438,438 ,692,678,684 }, {305,306,302 ,424,424,424 ,693,680,694 },
	{304,305,302 ,439,439,439 ,695,696,684 }, {302,303,304 ,440,440,440 ,694,697,698 },
	{274,275,276 ,409,409,409 ,699,700,701 }, {273,274,276 ,409,409,409 ,688,702,701 },
	{272,273,276 ,409,409,409 ,690,703,704 }, {271,272,276 ,409,409,409 ,705,706,707 },
	{280,271,276 ,409,409,409 ,708,709,710 }, {279,280,276 ,409,409,409 ,711,712,713 },
	{278,279,276 ,409,409,409 ,695,714,715 }, {276,277,278 ,409,409,409 ,713,697,695 },
	{286,285,284 ,441,441,441 ,716,717,689 }, {286,284,283 ,442,442,442 ,671,718,673 },
	{286,283,282 ,429,429,429 ,674,675,676 }, {286,282,281 ,443,443,443 ,674,677,719 },
	{286,281,290 ,444,444,444 ,671,720,680 }, {286,290,289 ,432,432,432 ,668,680,721 },
	{288,287,286 ,445,445,445 ,722,723,684 }, {289,288,286 ,446,446,446 ,721,695,724 },
	{351,349,203 ,447,448,449 ,725,726,727 }, {203,227,351 ,450,450,447 ,728,729,725 },
	{349,351,221 ,448,447,451 ,726,725,730 }, {201,349,221 ,452,448,452 ,731,726,732 },
	{155,159,351 ,453,453,447 ,733,734,725 }, {227,155,351 ,454,454,447 ,735,736,725 },
	{135,221,351 ,455,455,447 ,737,738,725 }, {351,159,135 ,447,456,456 ,725,739,740 },
	{350,212,203 ,457,458,458 ,741,742,727 }, {350,203,349 ,457,459,448 ,741,727,726 },
	{349,201,213 ,448,460,460 ,726,743,744 }, {350,349,213 ,457,448,461 ,741,726,745 },
	{82,212,350 ,462,462,457 ,746,747,741 }, {89,82,350 ,463,463,457 ,748,749,741 },
	{350,213,81 ,457,464,464 ,741,750,751 }, {81,89,350 ,465,465,457 ,752,753,741 },
	{359,358,361 ,466,467,468 ,754,755,756 }, {361,360,359 ,468,469,466 ,756,757,754 },
	{360,362,359 ,469,470,466 ,757,758,754 }, {361,358,363 ,468,467,471 ,756,755,759 },
	{357,363,358 ,472,471,467 ,760,759,755 }, {358,356,357 ,467,473,472 ,755,761,760 },
	{359,354,356 ,466,474,473 ,754,762,761 }, {358,359,356 ,467,466,473 ,755,754,761 },
	{353,354,359 ,475,474,466 ,763,762,754 }, {359,362,353 ,466,470,475 ,754,758,763 },
	{352,353,362 ,476,475,470 ,764,763,758 }, {362,360,352 ,470,469,476 ,758,757,764 },
	{360,361,355 ,469,468,477 ,757,756,765 }, {360,355,352 ,469,477,476 ,757,765,764 },
	{355,127,126 ,478,479,479 ,765,766,767 }, {357,355,126 ,480,478,480 ,760,765,768 },
	{363,125,128 ,471,481,481 ,759,769,770 }, {361,363,128 ,468,471,482 ,756,759,770 },
	{355,361,128 ,477,468,483 ,765,756,770 }, {128,127,355 ,484,484,477 ,771,766,765 },
	{126,125,363 ,485,485,471 ,768,772,759 }, {357,126,363 ,472,486,471 ,760,773,759 },
	{373,370,371 ,487,488,489 ,774,775,776 }, {371,372,373 ,489,490,487 ,776,777,774 },
	{374,372,371 ,491,490,489 ,778,777,776 }, {370,373,375 ,488,487,492 ,775,774,779 },
	{369,368,370 ,493,494,488 ,780,781,775 }, {370,375,369 ,488,492,493 ,775,779,780 },
	{371,370,368 ,489,488,494 ,776,775,781 }, {366,371,368 ,495,489,494 ,782,776,781 },
	{371,366,365 ,489,495,496 ,776,782,783 }, {365,374,371 ,496,491,489 ,783,778,776 },
	{374,365,364 ,491,496,497 ,778,783,784 }, {364,372,374 ,497,490,491 ,784,777,778 },
	{372,364,367 ,490,497,498 ,777,784,785 }, {367,373,372 ,498,487,490 ,785,774,777 },
	{367,369,129 ,499,500,500 ,785,780,786 }, {134,367,129 ,501,499,501 ,787,785,788 },
	{375,373,133 ,492,487,502 ,779,774,789 }, {136,375,133 ,503,492,503 ,790,779,791 },
	{133,373,367 ,504,487,498 ,792,774,785 }, {367,134,133 ,498,505,505 ,785,793,794 },
	{129,369,375 ,506,493,492 ,795,780,779 }, {136,129,375 ,507,507,492 ,796,797,779 },
	{354,353,352 ,508,508,508 ,798,799,800 }, {352,355,356 ,509,509,509 ,801,802,803 },
	{356,354,352 ,510,510,510 ,804,805,806 }, {357,356,355 ,511,511,511 ,807,808,809 },
	{365,366,364 ,512,512,512 ,810,811,812 }, {368,367,364 ,513,513,513 ,813,814,815 },
	{364,366,368 ,514,514,514 ,816,817,818 }, {368,369,367 ,515,515,515 ,819,820,821 }
};
static GLfloat vertices [376][3] = {
{0.165186f,0.474791f,-0.0481281f},{6.82545e-010f,0.495885f,-0.0290175f},{6.82545e-010f,0.5f,-0.0142633f},
{6.82545e-010f,0.475376f,-0.00424912f},{6.82545e-010f,0.460813f,0.0499119f},{0.173658f,0.457315f,-0.00424912f},
{0.169163f,0.447628f,0.0467705f},{0.172021f,0.47651f,-0.0142633f},{0.0979164f,0.492249f,-0.0303752f},
{0.0979164f,0.456938f,0.048799f},{0.0979164f,0.469004f,-0.00424912f},{0.0979164f,0.495259f,-0.0142633f},
{0.138411f,0.459367f,0.0771504f},{0.143925f,0.461848f,0.0731154f},{0.0979164f,0.468067f,0.0820374f},
{0.0979164f,0.47227f,0.0779256f},{6.82545e-010f,0.473036f,0.0858419f},{6.82545e-010f,0.477258f,0.0818202f},
{6.82545e-010f,0.474791f,-0.0481281f},{0.1745f,0.456356f,0.0280188f},{0.0979164f,0.46731f,0.0280188f},
{6.82545e-010f,0.473665f,0.0280188f},{0.0979164f,0.474791f,-0.0481281f},{0.168962f,0.475608f,-0.0342639f},
{-0.143925f,0.461848f,0.0731154f},{-0.169163f,0.447628f,0.0467705f},{-0.1745f,0.456356f,0.0280188f},
{-0.168962f,0.475608f,-0.0342639f},{-0.172021f,0.47651f,-0.0142633f},{-0.0979164f,0.495259f,-0.0142633f},
{-0.0979164f,0.492249f,-0.0303752f},{-0.0979164f,0.474791f,-0.0481281f},{-0.165186f,0.474791f,-0.0481281f},
{-0.173658f,0.457315f,-0.00424912f},{-0.0979164f,0.46731f,0.0280188f},{-0.0979164f,0.456938f,0.048799f},
{-0.0979164f,0.47227f,0.0779256f},{-0.0979164f,0.468067f,0.0820374f},{-0.0979164f,0.469004f,-0.00424912f},
{-0.138411f,0.459367f,0.0771504f},{0.113652f,-0.484107f,-0.115229f},{0.175542f,-0.455959f,-0.116803f},
{6.82545e-010f,-0.5f,-0.115229f},{6.82545e-010f,-0.49784f,-0.0220521f},{0.113652f,-0.481011f,-0.0220521f},
{0.175692f,-0.452228f,-0.0220521f},{6.82545e-010f,-0.498922f,-0.0387424f},{0.113652f,-0.482944f,-0.0387424f},
{0.17564f,-0.453133f,-0.0387424f},{6.82545e-010f,-0.489998f,-0.0769081f},{0.113652f,-0.474678f,-0.0769081f},
{0.1756f,-0.449583f,-0.0769081f},{-0.113652f,-0.474678f,-0.0769081f},{-0.113652f,-0.484107f,-0.115229f},
{-0.113652f,-0.482944f,-0.0387424f},{-0.113652f,-0.481011f,-0.0220521f},{-0.175542f,-0.455959f,-0.116803f},
{-0.1756f,-0.449583f,-0.0769081f},{-0.17564f,-0.453133f,-0.0387424f},{-0.175692f,-0.452228f,-0.0220521f},
{0.179706f,0.420507f,-0.074341f},{0.194947f,-0.246319f,-0.11948f},{0.195679f,0.0785289f,-0.118964f},
{0.20258f,-0.250915f,-0.0431137f},{0.202653f,-0.28237f,0.00854671f},{0.198699f,-0.389085f,0.0122553f},
{0.195726f,-0.41905f,-0.0220521f},{0.193732f,-0.428887f,-0.116438f},{0.193006f,0.359968f,-0.0876988f},
{0.199816f,0.34649f,-0.0141506f},{0.201632f,0.319262f,0.0186474f},{0.204742f,0.187807f,0.0100294f},
{0.205156f,0.164345f,-0.0421637f},{0.196921f,0.167964f,-0.118496f},{0.201309f,-0.340925f,0.0235529f},
{0.204608f,0.251463f,0.0337157f},{0.179283f,0.0785289f,0.0527684f},{0.188001f,0.392849f,0.034635f},
{0.145956f,0.362186f,0.0842424f},{0.181433f,-0.399742f,0.0203798f},{0.114329f,0.172678f,0.132511f},
{0.168351f,-0.0200177f,0.0487904f},{0.125132f,0.0345616f,0.116998f},{0.113355f,0.0785289f,0.133404f},
{0.182361f,-0.343756f,0.0340641f},{0.202703f,-0.125796f,-0.00258575f},{0.189846f,0.407958f,-0.0142633f},
{0.203782f,0.0785289f,-0.000348677f},{0.126131f,0.250598f,0.116019f},{0.147685f,0.0070092f,0.0841593f},
{0.156389f,0.0785289f,0.0850938f},{0.160729f,0.216005f,0.0850137f},{0.177578f,0.230452f,0.064726f},
{0.178212f,-0.434672f,-0.0015253f},{0.1462f,0.410945f,0.0734128f},{0.195063f,-0.424185f,-0.0387424f},
{0.197011f,-0.409943f,-0.00458018f},{0.194562f,-0.424973f,-0.0769081f},{0.172141f,0.376835f,0.0660285f},
{0.179417f,-0.216333f,0.0463776f},{0.193639f,-0.216333f,0.0372182f},{0.200759f,-0.216333f,0.0259474f},
{0.202682f,-0.216333f,0.00529888f},{0.195044f,-0.216333f,-0.119412f},{0.201454f,-0.216333f,-0.0445343f},
{0.202766f,0.0785289f,-0.0500203f},{0.19668f,-0.0371321f,-0.0556145f},{0.197679f,-0.0371321f,-0.00702947f},
{0.189863f,-0.0371321f,-0.119142f},{0.195308f,-0.0828239f,0.0219538f},{0.192112f,-0.0346377f,0.0205794f},
{0.198765f,0.0785289f,0.0275055f},{0.1277f,0.0785289f,0.11921f},{0.131485f,0.185502f,0.118453f},
{0.189492f,0.400397f,0.0142591f},{0.194872f,0.242556f,0.0504878f},{0.190698f,0.312626f,0.0508879f},
{0.183477f,0.384156f,0.0516772f},{0.187335f,0.415723f,-0.0452362f},{0.198802f,0.357849f,-0.0502311f},
{0.191249f,0.0785289f,0.0429187f},{0.182253f,-0.0290117f,0.0372707f},{0.18193f,-0.0484389f,0.0369062f},
{0.198082f,0.316501f,0.0341385f},{0.199323f,-0.136333f,-0.0500769f},{0.173057f,-0.0148871f,0.0455924f},
{0.171151f,-0.00242974f,0.0488021f},{0.179176f,-0.00173625f,0.0422881f},{0.175483f,-0.0162453f,0.0434169f},
{-0.171151f,-0.00242974f,0.0488021f},{-0.179283f,0.0785289f,0.0527684f},{-0.191249f,0.0785289f,0.0429187f},
{-0.182253f,-0.0290117f,0.0372707f},{-0.175483f,-0.0162453f,0.0434169f},{-0.179176f,-0.00173625f,0.0422881f},
{-0.168351f,-0.0200177f,0.0487904f},{-0.173057f,-0.0148871f,0.0455924f},{-0.199323f,-0.136333f,-0.0500769f},
{-0.201454f,-0.216333f,-0.0445343f},{-0.202703f,-0.125796f,-0.00258575f},{-0.198082f,0.316501f,0.0341385f},
{-0.204608f,0.251463f,0.0337157f},{-0.190698f,0.312626f,0.0508879f},{-0.201632f,0.319262f,0.0186474f},
{-0.193639f,-0.216333f,0.0372182f},{-0.18193f,-0.0484389f,0.0369062f},{-0.198765f,0.0785289f,0.0275055f},
{-0.192112f,-0.0346377f,0.0205794f},{-0.194872f,0.242556f,0.0504878f},{-0.145956f,0.362186f,0.0842424f},
{-0.160729f,0.216005f,0.0850137f},{-0.131485f,0.185502f,0.118453f},{-0.1277f,0.0785289f,0.11921f},
{-0.113355f,0.0785289f,0.133404f},{-0.114329f,0.172678f,0.132511f},{-0.125132f,0.0345616f,0.116998f},
{-0.177578f,0.230452f,0.064726f},{-0.126131f,0.250598f,0.116019f},{-0.156389f,0.0785289f,0.0850938f},
{-0.147685f,0.0070092f,0.0841593f},{-0.1462f,0.410945f,0.0734128f},{-0.188001f,0.392849f,0.034635f},
{-0.179706f,0.420507f,-0.074341f},{-0.187335f,0.415723f,-0.0452362f},{-0.198802f,0.357849f,-0.0502311f},
{-0.199816f,0.34649f,-0.0141506f},{-0.189846f,0.407958f,-0.0142633f},{-0.193006f,0.359968f,-0.0876988f},
{-0.189492f,0.400397f,0.0142591f},{-0.183477f,0.384156f,0.0516772f},{-0.172141f,0.376835f,0.0660285f},
{-0.201309f,-0.340925f,0.0235529f},{-0.200759f,-0.216333f,0.0259474f},{-0.195308f,-0.0828239f,0.0219538f},
{-0.203782f,0.0785289f,-0.000348677f},{-0.197679f,-0.0371321f,-0.00702947f},{-0.202682f,-0.216333f,0.00529888f},
{-0.202766f,0.0785289f,-0.0500203f},{-0.19668f,-0.0371321f,-0.0556145f},{-0.189863f,-0.0371321f,-0.119142f},
{-0.195679f,0.0785289f,-0.118964f},{-0.195044f,-0.216333f,-0.119412f},{-0.205156f,0.164345f,-0.0421637f},
{-0.196921f,0.167964f,-0.118496f},{-0.179417f,-0.216333f,0.0463776f},{-0.182361f,-0.343756f,0.0340641f},
{-0.202653f,-0.28237f,0.00854671f},{-0.194947f,-0.246319f,-0.11948f},{-0.20258f,-0.250915f,-0.0431137f},
{-0.193732f,-0.428887f,-0.116438f},{-0.194562f,-0.424973f,-0.0769081f},{-0.195063f,-0.424185f,-0.0387424f},
{-0.197011f,-0.409943f,-0.00458018f},{-0.195726f,-0.41905f,-0.0220521f},{-0.198699f,-0.389085f,0.0122553f},
{-0.178212f,-0.434672f,-0.0015253f},{-0.181433f,-0.399742f,0.0203798f},{-0.204742f,0.187807f,0.0100294f},
{6.82545e-010f,0.168259f,0.14461f},{6.82545e-010f,-0.413985f,0.0312275f},{6.82545e-010f,-0.347442f,0.0481428f},
{6.82545e-010f,-0.122232f,0.0537508f},{6.82545e-010f,0.0785289f,0.145174f},{6.82545e-010f,0.0178893f,0.131762f},
{6.82545e-010f,0.372678f,0.0931505f},{0.113175f,-0.408287f,0.0274473f},{6.82545e-010f,0.253134f,0.126524f},
{0.0979164f,0.369609f,0.0893649f},{0.09059f,0.251941f,0.121428f},{6.82545e-010f,-0.467268f,0.00539864f},
{0.113453f,-0.45335f,0.00187233f},{0.087295f,0.169877f,0.139978f},{0.0869415f,0.0287049f,0.127871f},
{0.0955095f,-0.0957936f,0.050985f},{0.0821373f,0.0785289f,0.141732f},{0.112678f,-0.345886f,0.042661f},
{6.82545e-010f,0.415157f,0.0804352f},{0.0979164f,0.413526f,0.0771871f},{0.108699f,-0.216333f,0.0524797f},
{6.82545e-010f,-0.216333f,0.0577729f},{-0.108699f,-0.216333f,0.0524797f},{-0.0955095f,-0.0957936f,0.050985f},
{-0.112678f,-0.345886f,0.042661f},{-0.0979164f,0.413526f,0.0771871f},{-0.0979164f,0.369609f,0.0893649f},
{-0.09059f,0.251941f,0.121428f},{-0.087295f,0.169877f,0.139978f},{-0.0869415f,0.0287049f,0.127871f},
{-0.0821373f,0.0785289f,0.141732f},{-0.113175f,-0.408287f,0.0274473f},{-0.113453f,-0.45335f,0.00187233f},
{0.196303f,-0.385644f,-0.00357175f},{0.196303f,-0.414079f,-0.0427096f},{0.196303f,-0.414079f,-0.0910866f},
{0.196303f,-0.385644f,-0.130224f},{0.196303f,-0.339635f,-0.145174f},{0.196303f,-0.293625f,-0.130224f},
{0.196303f,-0.26519f,-0.0910866f},{0.196303f,-0.26519f,-0.0427096f},{0.196303f,-0.293625f,-0.00357175f},
{0.196303f,-0.339635f,0.0113777f},{0.132778f,-0.385644f,-0.00357175f},{0.132778f,-0.414079f,-0.0427096f},
{0.132778f,-0.414079f,-0.0910866f},{0.132778f,-0.385644f,-0.130224f},{0.132778f,-0.339635f,-0.145174f},
{0.132778f,-0.293625f,-0.130224f},{0.132778f,-0.26519f,-0.0910866f},{0.132778f,-0.26519f,-0.0427096f},
{0.132778f,-0.293625f,-0.00357175f},{0.132778f,-0.339635f,0.0113777f},{-0.196303f,-0.339635f,-0.145174f},
{-0.196303f,-0.385644f,-0.130224f},{-0.196303f,-0.414079f,-0.0910866f},{-0.196303f,-0.414079f,-0.0427096f},
{-0.196303f,-0.385644f,-0.00357175f},{-0.196303f,-0.339635f,0.0113777f},{-0.196303f,-0.293625f,-0.00357175f},
{-0.196303f,-0.26519f,-0.0427096f},{-0.196303f,-0.26519f,-0.0910866f},{-0.196303f,-0.293625f,-0.130224f},
{-0.132778f,-0.339635f,-0.145174f},{-0.132778f,-0.293625f,-0.130224f},{-0.132778f,-0.26519f,-0.0910866f},
{-0.132778f,-0.26519f,-0.0427096f},{-0.132778f,-0.293625f,-0.00357175f},{-0.132778f,-0.339635f,0.0113777f},
{-0.132778f,-0.385644f,-0.00357175f},{-0.132778f,-0.414079f,-0.0427096f},{-0.132778f,-0.414079f,-0.0910866f},
{-0.132778f,-0.385644f,-0.130224f},{0.196303f,0.212981f,-0.00357175f},{0.196303f,0.184546f,-0.0427096f},
{0.196303f,0.184546f,-0.0910866f},{0.196303f,0.212981f,-0.130224f},{0.196303f,0.258991f,-0.145174f},
{0.196303f,0.305f,-0.130224f},{0.196303f,0.333435f,-0.0910866f},{0.196303f,0.333435f,-0.0427096f},
{0.196303f,0.305f,-0.00357175f},{0.196303f,0.258991f,0.0113777f},{0.132778f,0.212981f,-0.00357175f},
{0.132778f,0.184546f,-0.0427096f},{0.132778f,0.184546f,-0.0910866f},{0.132778f,0.212981f,-0.130224f},
{0.132778f,0.258991f,-0.145174f},{0.132778f,0.305f,-0.130224f},{0.132778f,0.333435f,-0.0910866f},
{0.132778f,0.333435f,-0.0427096f},{0.132778f,0.305f,-0.00357175f},{0.132778f,0.258991f,0.0113777f},
{-0.196303f,0.258991f,-0.145174f},{-0.196303f,0.212981f,-0.130224f},{-0.196303f,0.184546f,-0.0910866f},
{-0.196303f,0.184546f,-0.0427096f},{-0.196303f,0.212981f,-0.00357175f},{-0.196303f,0.258991f,0.0113777f},
{-0.196303f,0.305f,-0.00357175f},{-0.196303f,0.333435f,-0.0427096f},{-0.196303f,0.333435f,-0.0910866f},
{-0.196303f,0.305f,-0.130224f},{-0.132778f,0.258991f,-0.145174f},{-0.132778f,0.305f,-0.130224f},
{-0.132778f,0.333435f,-0.0910866f},{-0.132778f,0.333435f,-0.0427096f},{-0.132778f,0.305f,-0.00357175f},
{-0.132778f,0.258991f,0.0113777f},{-0.132778f,0.212981f,-0.00357175f},{-0.132778f,0.184546f,-0.0427096f},
{-0.132778f,0.184546f,-0.0910866f},{-0.132778f,0.212981f,-0.130224f},{0.102952f,0.359968f,-0.0876988f},
{0.0979164f,0.420507f,-0.074341f},{-0.102952f,0.359968f,-0.0876988f},{-0.0979164f,0.420507f,-0.074341f},
{0.0973887f,0.167964f,-0.118496f},{0.102952f,0.34649f,-0.0141506f},{0.102952f,0.319262f,0.0186474f},
{0.102952f,0.251463f,0.0337157f},{0.102952f,0.164345f,-0.0421637f},{0.102952f,0.187807f,0.0100294f},
{0.102952f,-0.250915f,-0.0431137f},{0.0973887f,-0.246319f,-0.11948f},{0.102952f,-0.28237f,0.00854671f},
{0.102952f,-0.340925f,0.0235529f},{0.102952f,-0.428887f,-0.116438f},{0.102952f,-0.41905f,-0.0220521f},
{0.102952f,-0.389085f,0.0122553f},{0.102952f,-0.424185f,-0.0387424f},{0.102952f,-0.409943f,-0.00458018f},
{0.102952f,-0.424973f,-0.0769081f},{0.102952f,0.357849f,-0.0502311f},{-0.0973887f,0.167964f,-0.118496f},
{-0.102952f,0.164345f,-0.0421637f},{-0.102952f,0.187807f,0.0100294f},{-0.102952f,0.251463f,0.0337157f},
{-0.102952f,0.319262f,0.0186474f},{-0.102952f,0.34649f,-0.0141506f},{-0.102952f,0.357849f,-0.0502311f},
{-0.0973887f,-0.246319f,-0.11948f},{-0.102952f,-0.428887f,-0.116438f},{-0.102952f,-0.424973f,-0.0769081f},
{-0.102952f,-0.424185f,-0.0387424f},{-0.102952f,-0.41905f,-0.0220521f},{-0.102952f,-0.409943f,-0.00458018f},
{-0.102952f,-0.389085f,0.0122553f},{-0.102952f,-0.340925f,0.0235529f},{-0.102952f,-0.28237f,0.00854671f},
{-0.102952f,-0.250915f,-0.0431137f},{6.82545e-010f,-0.0606863f,0.0911214f},{0.092047f,-0.0402891f,0.0877739f},
{-0.092047f,-0.0402891f,0.0877739f},{0.210049f,0.00768499f,0.0486612f},{0.218158f,0.00945784f,0.0583728f},
{0.209492f,0.00768499f,0.0771623f},{0.181881f,-0.00163993f,0.0461887f},{0.180826f,-0.00159149f,0.076606f},
{0.174039f,-0.0023697f,0.0527458f},{0.183369f,-0.0136513f,0.0719413f},{0.206381f,-0.00588737f,0.0719413f},
{0.20859f,-0.00854663f,0.0537659f},{0.182168f,-0.0163105f,0.0528405f},{0.213542f,-0.00371702f,0.0583728f},
{0.176852f,-0.0150878f,0.0505329f},{-0.210049f,0.00768499f,0.0486612f},{-0.218158f,0.00945784f,0.0583728f},
{-0.209492f,0.00768499f,0.0771623f},{-0.181881f,-0.00163993f,0.0461887f},{-0.180826f,-0.00159149f,0.076606f},
{-0.174039f,-0.0023697f,0.0527458f},{-0.183369f,-0.0136513f,0.0719413f},{-0.206381f,-0.00588737f,0.0719413f},
{-0.20859f,-0.00854663f,0.0537659f},{-0.182168f,-0.0163105f,0.0528405f},{-0.213542f,-0.00371702f,0.0583728f},
{-0.176852f,-0.0150878f,0.0505329f}
};
static GLfloat normals [516][3] = {
{-0.123882f,0.915146f,0.383615f},{-0.0853064f,0.882605f,-0.46231f},{-0.226851f,0.965755f,-0.125917f},
{-0.161105f,0.70889f,0.686673f},{0.0f,0.712992f,-0.701172f},{-0.117065f,0.892986f,-0.434595f},
{-9.56645e-010f,0.962469f,0.271391f},{-0.114523f,0.961435f,0.250055f},{-0.0963279f,0.995241f,0.0147253f},
{-1.04734e-009f,0.987664f,-0.156587f},{-0.137362f,0.960338f,0.242658f},{-0.131853f,0.974468f,0.181734f},
{-0.140139f,0.955445f,0.25978f},{-4.65967e-009f,0.98621f,0.165498f},{0.0f,0.694539f,0.719455f},
{-0.147177f,0.691688f,0.707041f},{-2.3294e-010f,0.69249f,-0.721428f},{-1.29571e-009f,0.909865f,-0.414904f},
{-2.88902e-009f,0.670601f,0.741818f},{-0.0916608f,0.816407f,0.570156f},{-2.49734e-009f,0.897524f,0.440965f},
{-0.193082f,0.979682f,-0.0542524f},{-0.227498f,0.671582f,0.70514f},{-0.138056f,0.921882f,0.362042f},
{0.123882f,0.915146f,0.383615f},{0.161105f,0.70889f,0.686673f},{0.226851f,0.965755f,-0.125917f},
{0.0853064f,0.882605f,-0.46231f},{0.117065f,0.892985f,-0.434595f},{0.0963279f,0.995241f,0.0147253f},
{0.114523f,0.961435f,0.250055f},{0.131853f,0.974468f,0.181734f},{0.137362f,0.960338f,0.242658f},
{0.140139f,0.955445f,0.25978f},{0.147177f,0.691688f,0.707041f},{0.0916608f,0.816407f,0.570156f},
{0.193082f,0.979682f,-0.0542524f},{0.227498f,0.671582f,0.70514f},{0.138056f,0.921882f,0.362042f},
{-0.263244f,-0.940404f,0.215273f},{0.0f,-0.967584f,0.252551f},{5.14022e-009f,-0.995925f,0.090181f},
{-0.23324f,-0.971763f,-0.0357079f},{5.05908e-009f,-0.992019f,-0.126088f},{-0.280753f,-0.959124f,-0.0354938f},
{-3.76477e-009f,-0.995951f,0.0898947f},{-0.33566f,-0.937732f,0.0893878f},{-0.400612f,-0.915996f,0.0214897f},
{-0.412754f,-0.899523f,0.14315f},{-0.429111f,-0.902968f,0.0226323f},{-0.42038f,-0.906085f,0.0478578f},
{0.263244f,-0.940404f,0.215273f},{0.23324f,-0.971763f,-0.0357079f},{0.280753f,-0.959124f,-0.0354938f},
{0.33566f,-0.937732f,0.0893878f},{0.412753f,-0.899524f,0.14315f},{0.400612f,-0.915996f,0.0214897f},
{0.429111f,-0.902968f,0.0226323f},{0.42038f,-0.906085f,0.0478578f},{-0.728957f,-0.0742548f,0.68052f},
{-0.724607f,-0.0818318f,0.684287f},{-0.632249f,-0.101139f,0.768135f},{-0.609838f,-0.0785027f,0.788629f},
{-0.588615f,-0.0917377f,0.803191f},{-0.726853f,-0.0661276f,0.683602f},{-0.657316f,-0.0529929f,0.75175f},
{-0.63225f,-0.101138f,0.768135f},{-0.997718f,0.0160866f,-0.0655755f},{-0.998641f,0.0310028f,0.0419023f},
{-0.99589f,0.0227443f,-0.0876657f},{-0.932884f,0.0298643f,0.358937f},{-0.826518f,0.0679329f,0.558796f},
{-0.937105f,0.106687f,0.332344f},{-0.976452f,0.131843f,0.170761f},{-0.772024f,0.0262658f,0.635051f},
{-0.728798f,0.0193228f,0.684456f},{-0.938663f,-0.0577294f,0.33997f},{-0.928439f,-0.0164532f,0.371121f},
{-0.770899f,-0.0183543f,0.636693f},{-0.765258f,0.013016f,0.643592f},{-0.698029f,0.0292835f,0.71547f},
{-0.64475f,0.0884217f,0.759263f},{-0.648521f,0.0204203f,0.760923f},{-0.716786f,-0.0388667f,0.696209f},
{-0.702837f,-0.0386094f,0.710302f},{-0.748336f,-0.0855327f,0.657782f},{-0.676317f,0.00849477f,0.736561f},
{-0.642603f,0.0813415f,0.761869f},{-0.783807f,-0.0596709f,0.618132f},{-0.8026f,-0.107547f,0.586742f},
{-0.731435f,0.0366624f,0.680924f},{-0.592839f,0.125185f,0.795532f},{-0.902656f,0.229799f,0.363873f},
{-0.944162f,0.16832f,0.283244f},{-0.958241f,0.257906f,-0.123525f},{-0.92813f,0.319692f,-0.190714f},
{-0.923495f,0.313643f,-0.220875f},{-0.98633f,0.15718f,0.0494698f},{-0.973117f,0.218694f,0.0722262f},
{-0.981072f,0.187411f,-0.0487366f},{-0.95809f,0.22877f,-0.172417f},{-0.967704f,0.251619f,-0.0153842f},
{-0.974913f,0.207687f,0.0800647f},{-0.963396f,0.220663f,0.152234f},{-0.935654f,0.315891f,-0.157369f},
{-0.920642f,0.239382f,0.308406f},{-0.631998f,0.125278f,0.764777f},{-0.711316f,0.212204f,0.670074f},
{-0.85056f,0.154915f,0.502542f},{-0.951613f,0.293908f,0.0897274f},{-0.963585f,0.0178288f,0.266806f},
{-0.725528f,-0.0772215f,0.683846f},{-0.904422f,0.231647f,0.358273f},{-0.91969f,0.0611916f,0.387849f},
{-0.995231f,-0.0409969f,0.0885126f},{-0.99625f,0.00408534f,0.0864193f},{-0.999604f,0.00813518f,0.0269293f},
{-0.997453f,-0.0346077f,-0.0623709f},{-0.997357f,-0.00839502f,-0.0721764f},{-0.99596f,0.0155088f,-0.088452f},
{-0.994709f,-0.000110358f,-0.102729f},{-0.993932f,-0.0368058f,-0.10366f},{-0.999088f,-0.0179328f,-0.0387468f},
{-0.994334f,-0.0155368f,-0.10516f},{-0.997989f,0.00203332f,0.0633546f},{-0.504621f,-0.16942f,0.846554f},
{-0.525276f,-0.0370626f,0.850124f},{-0.998946f,0.0198172f,-0.0414015f},{-0.995722f,0.0126828f,-0.091521f},
{-0.54567f,0.241265f,0.802518f},{-0.82759f,-0.560758f,0.025403f},{-0.791917f,-0.610196f,0.0229813f},
{-0.792124f,-0.610356f,0.00221396f},{-0.829251f,-0.556393f,-0.0526311f},{-0.791368f,-0.609773f,0.0437388f},
{-0.809438f,-0.58351f,0.0657779f},{-0.826291f,-0.55637f,0.0877182f},{-0.830084f,-0.556953f,0.0276433f},
{-0.849426f,-0.512909f,0.124095f},{-0.819257f,-0.526026f,0.228287f},{-0.581216f,0.0307223f,0.813169f},
{-0.811215f,-0.489836f,0.31936f},{-0.768481f,-0.50629f,0.391288f},{-0.701451f,-0.521268f,0.486052f},
{-0.609575f,-0.398987f,0.685002f},{-0.551435f,-0.327085f,0.767421f},{-0.994811f,-0.0243304f,0.0987901f},
{-0.892063f,0.366374f,-0.264564f},{-0.964715f,0.262723f,-0.0173837f},{0.724607f,-0.0818318f,0.684287f},
{0.728957f,-0.0742548f,0.68052f},{0.632249f,-0.101139f,0.768135f},{0.588615f,-0.0917377f,0.803191f},
{0.609838f,-0.0785027f,0.788629f},{0.726853f,-0.0661276f,0.683602f},{0.657316f,-0.0529929f,0.75175f},
{0.63225f,-0.101138f,0.768135f},{0.998641f,0.0310028f,0.0419023f},{0.997718f,0.0160867f,-0.0655754f},
{0.99589f,0.0227443f,-0.0876657f},{0.826518f,0.0679328f,0.558796f},{0.932884f,0.0298643f,0.358937f},
{0.937105f,0.106687f,0.332344f},{0.976452f,0.131843f,0.170761f},{0.772024f,0.0262658f,0.635051f},
{0.728798f,0.0193228f,0.684456f},{0.938663f,-0.0577294f,0.33997f},{0.928439f,-0.0164532f,0.371121f},
{0.770899f,-0.0183543f,0.636693f},{0.698029f,0.0292835f,0.71547f},{0.765258f,0.013016f,0.643592f},
{0.64475f,0.0884217f,0.759263f},{0.648521f,0.0204203f,0.760923f},{0.702837f,-0.0386094f,0.710302f},
{0.716786f,-0.0388667f,0.696209f},{0.748336f,-0.0855327f,0.657782f},{0.676317f,0.00849477f,0.736561f},
{0.642603f,0.0813415f,0.761869f},{0.783807f,-0.0596709f,0.618132f},{0.8026f,-0.107547f,0.586742f},
{0.592839f,0.125185f,0.795532f},{0.731435f,0.0366624f,0.680924f},{0.902656f,0.229799f,0.363873f},
{0.944162f,0.16832f,0.283244f},{0.92813f,0.319692f,-0.190714f},{0.958241f,0.257906f,-0.123525f},
{0.923495f,0.313643f,-0.220875f},{0.973117f,0.218694f,0.0722264f},{0.98633f,0.15718f,0.0494698f},
{0.981072f,0.187411f,-0.0487364f},{0.95809f,0.22877f,-0.172417f},{0.967704f,0.251619f,-0.0153835f},
{0.974913f,0.207687f,0.0800647f},{0.963396f,0.220663f,0.152234f},{0.935654f,0.315891f,-0.157368f},
{0.920642f,0.239382f,0.308406f},{0.711315f,0.212205f,0.670075f},{0.631998f,0.125278f,0.764777f},
{0.85056f,0.154915f,0.502542f},{0.951613f,0.293908f,0.0897274f},{0.963585f,0.0178288f,0.266806f},
{0.725528f,-0.0772215f,0.683846f},{0.904423f,0.231648f,0.358272f},{0.91969f,0.0611916f,0.387849f},
{0.995231f,-0.0409969f,0.0885126f},{0.99625f,0.00408534f,0.0864193f},{0.999604f,0.00813518f,0.0269293f},
{0.997453f,-0.0346077f,-0.0623709f},{0.997357f,-0.00839502f,-0.0721764f},{0.99596f,0.015509f,-0.088452f},
{0.994709f,-0.000110358f,-0.102729f},{0.993932f,-0.0368058f,-0.10366f},{0.994334f,-0.0155368f,-0.10516f},
{0.999088f,-0.0179328f,-0.0387468f},{0.997989f,0.00203332f,0.0633546f},{0.525276f,-0.0370626f,0.850124f},
{0.504621f,-0.16942f,0.846554f},{0.998946f,0.0198172f,-0.0414014f},{0.995722f,0.0126831f,-0.0915208f},
{0.54567f,0.241265f,0.802518f},{0.82759f,-0.560758f,0.0254031f},{0.829251f,-0.556393f,-0.0526307f},
{0.791917f,-0.610196f,0.0229813f},{0.792125f,-0.610356f,0.00221395f},{0.826292f,-0.55637f,0.0877181f},
{0.809438f,-0.58351f,0.0657778f},{0.791368f,-0.609773f,0.0437388f},{0.830084f,-0.556953f,0.0276433f},
{0.819257f,-0.526026f,0.228287f},{0.849426f,-0.512909f,0.124095f},{0.581216f,0.0307223f,0.813169f},
{0.701451f,-0.521268f,0.486052f},{0.768481f,-0.50629f,0.391288f},{0.811215f,-0.489836f,0.31936f},
{0.551435f,-0.327085f,0.767421f},{0.609575f,-0.398987f,0.685002f},{0.994811f,-0.0243304f,0.0987901f},
{0.892062f,0.366375f,-0.264565f},{0.964715f,0.262723f,-0.0173837f},{-4.66052e-010f,0.0289032f,0.999582f},
{-0.0504266f,0.00895871f,0.998688f},{-0.0572595f,-0.0234186f,0.998085f},{9.33994e-010f,-0.0153012f,0.999883f},
{-0.0946136f,-0.158404f,0.982831f},{4.67901e-010f,-0.158118f,0.98742f},{-0.085741f,-0.0726751f,0.993663f},
{-0.124705f,-0.092581f,0.987865f},{-0.0405012f,-0.0099967f,0.999129f},{-0.085948f,0.0217561f,0.996062f},
{-0.0315512f,-0.0885356f,0.995573f},{-0.0640292f,0.0979529f,0.993129f},{-6.31427e-010f,0.0356211f,0.999365f},
{-0.0339466f,-0.092954f,0.995092f},{-0.102109f,0.251285f,0.962512f},{-9.32397e-010f,0.270797f,0.962637f},
{-0.155372f,0.110728f,0.981631f},{-1.24866e-009f,0.0773085f,0.997007f},{-1.86612e-009f,0.231944f,0.972729f},
{-0.118891f,0.231371f,0.965574f},{-0.121366f,-0.115923f,0.985816f},{-0.210058f,-0.254256f,0.94405f},
{-1.86659e-009f,-0.238638f,0.971109f},{0.0f,-0.105496f,0.99442f},{-0.10388f,-0.375984f,0.920785f},
{1.87836e-009f,-0.394467f,0.91891f},{-0.0984451f,-0.664859f,0.740453f},{1.25764e-009f,-0.590747f,0.806857f},
{-0.162666f,-0.581691f,0.796979f},{-0.103166f,-0.65025f,0.752683f},{-0.100663f,-0.0881298f,0.99101f},
{-0.0743224f,-0.0648581f,0.995123f},{-0.136231f,0.214151f,0.967254f},{-0.0892529f,0.266145f,0.959792f},
{-0.250399f,-0.237f,0.938686f},{-0.301186f,-0.261516f,0.917004f},{-0.26564f,0.00640369f,0.964051f},
{-0.25774f,0.00399487f,0.966206f},{-0.248688f,-0.750953f,0.611739f},{-0.289914f,-0.624879f,0.724898f},
{-0.18454f,-0.484247f,0.855249f},{-0.153013f,-0.535117f,0.830805f},{-0.24085f,0.204787f,0.948711f},
{-0.154667f,0.182098f,0.97104f},{-0.140527f,0.247985f,0.958517f},{-0.154908f,0.250737f,0.955581f},
{-0.129061f,-0.233919f,0.963652f},{-0.126089f,-0.237512f,0.963167f},{0.0572595f,-0.0234186f,0.998085f},
{0.0504266f,0.00895871f,0.998688f},{0.0946136f,-0.158404f,0.982831f},{0.085741f,-0.0726751f,0.993663f},
{0.124705f,-0.092581f,0.987865f},{0.0405012f,-0.0099967f,0.999129f},{0.085948f,0.0217561f,0.996062f},
{0.0339466f,-0.092954f,0.995092f},{0.0640292f,0.0979529f,0.993129f},{0.0315512f,-0.0885356f,0.995573f},
{0.102109f,0.251285f,0.962512f},{0.155372f,0.110728f,0.981631f},{0.118891f,0.231371f,0.965574f},
{0.121366f,-0.115923f,0.985816f},{0.210058f,-0.254256f,0.94405f},{0.10388f,-0.375984f,0.920785f},
{0.0984451f,-0.664859f,0.740453f},{0.103166f,-0.65025f,0.752683f},{0.162666f,-0.581691f,0.796979f},
{0.0743224f,-0.0648581f,0.995123f},{0.100663f,-0.0881298f,0.99101f},{0.0892529f,0.266145f,0.959792f},
{0.136231f,0.214151f,0.967254f},{0.250399f,-0.237f,0.938686f},{0.301186f,-0.261516f,0.917004f},
{0.26564f,0.00640369f,0.964051f},{0.25774f,0.00399487f,0.966206f},{0.289914f,-0.624879f,0.724898f},
{0.248688f,-0.750953f,0.611739f},{0.18454f,-0.484247f,0.855249f},{0.153013f,-0.535117f,0.830805f},
{0.154667f,0.182098f,0.97104f},{0.24085f,0.204787f,0.948711f},{0.140527f,0.247985f,0.958517f},
{0.154908f,0.250737f,0.955581f},{0.126089f,-0.237512f,0.963167f},{0.129061f,-0.233919f,0.963652f},
{0.0f,-0.309019f,0.951056f},{0.0f,0.309019f,0.951056f},{0.0f,0.809017f,0.587785f},
{0.0f,1.0f,4.07826e-007f},{0.0f,0.809017f,-0.587785f},{0.0f,0.309019f,-0.951056f},
{0.0f,-0.309019f,-0.951056f},{0.0f,-0.809017f,-0.587785f},{0.0f,-1.0f,0.0f},
{0.0f,-0.809017f,0.587785f},{0.0f,-0.309018f,0.951056f},{0.0f,0.809017f,0.587785f},
{0.0f,1.0f,0.0f},{0.0f,0.809017f,-0.587786f},{0.0f,-0.309019f,-0.951056f},
{0.0f,-0.809017f,-0.587785f},{0.0f,-0.809017f,0.587785f},{0.0f,0.434845f,-0.900505f},
{0.0f,0.363904f,-0.931436f},{0.0f,0.343097f,-0.9393f},{0.0f,0.215464f,-0.976512f},
{2.67423e-008f,0.215464f,-0.976512f},{-3.56564e-008f,0.215464f,-0.976512f},{-1.06969e-007f,0.215464f,-0.976512f},
{1.06969e-007f,0.215464f,-0.976512f},{0.308604f,-0.000435779f,-0.95119f},{0.0f,0.00523249f,-0.999986f},
{-0.433396f,-0.00288623f,-0.901199f},{0.0f,0.241303f,-0.97045f},{0.0f,0.0135085f,-0.999909f},
{0.0f,0.23456f,-0.972102f},{0.0f,0.00227542f,-0.999997f},{0.0f,-0.245237f,-0.969463f},
{0.0f,-0.240972f,-0.970532f},{0.0f,0.0f,-1.0f},{0.0f,-0.055838f,-0.99844f},
{0.0f,0.00150287f,-0.999999f},{0.0f,0.00154087f,-0.999999f},{-0.997149f,-0.0171056f,-0.0734874f},
{-1.0f,4.56504e-008f,-2.05211e-008f},{-1.0f,3.41691e-008f,-4.07469e-008f},{-1.0f,2.26878e-008f,-6.09727e-008f},
{-1.0f,-4.86916e-008f,-8.41385e-008f},{-1.0f,0.0f,0.0f},{-1.0f,-1.08727e-007f,-1.37791e-007f},
{-1.0f,-2.17454e-007f,-2.75581e-007f},{-6.18399e-009f,-0.998405f,-0.0564531f},{-2.07555e-009f,-0.990336f,-0.138691f},
{0.0f,-0.998405f,-0.0564531f},{0.0f,-0.95384f,-0.300315f},{0.0f,-0.907471f,-0.420115f},
{-0.997057f,0.0292024f,-0.0708804f},{0.0f,0.995135f,-0.0985231f},{0.0f,0.99736f,-0.0726181f},
{0.0f,0.999787f,-0.0206608f},{0.0f,0.993622f,-0.112761f},{0.0f,0.886777f,-0.462197f},
{0.0f,0.815802f,-0.578331f},{0.0f,0.936224f,-0.351403f},{0.0f,0.628079f,-0.778149f},
{0.0f,0.506104f,-0.862472f},{0.0f,0.955785f,-0.294068f},{0.0f,-0.769423f,-0.638739f},
{0.0f,-0.614585f,-0.788851f},{0.0f,-0.029463f,-0.999566f},{0.0f,-0.216955f,-0.976182f},
{0.0f,0.998878f,0.0473599f},{0.0f,0.982466f,-0.186444f},{0.0f,0.912083f,-0.410006f},
{0.0f,0.575968f,-0.817472f},{0.0f,0.348738f,-0.93722f},{0.0f,-0.998195f,-0.0600646f},
{0.0f,-0.975695f,-0.219133f},{0.0f,-0.854123f,-0.520071f},{-5.88791e-008f,-0.595084f,-0.803663f},
{-1.69018e-007f,-0.854123f,-0.520071f},{-4.9126e-008f,-0.248255f,-0.968695f},{0.0f,0.0713286f,-0.997453f},
{0.0f,-0.248255f,-0.968695f},{0.0f,0.228384f,-0.973571f},{0.997149f,-0.0171055f,-0.0734875f},
{1.0f,6.83382e-008f,-8.14939e-008f},{1.0f,9.13008e-008f,-4.10422e-008f},{1.0f,-9.73832e-008f,-1.68277e-007f},
{1.0f,4.53756e-008f,-1.21945e-007f},{1.0f,0.0f,0.0f},{1.0f,-2.17454e-007f,-2.75581e-007f},
{1.0f,-4.34908e-007f,-5.51163e-007f},{2.07555e-009f,-0.990336f,-0.138691f},{6.18399e-009f,-0.998405f,-0.0564531f},
{0.0f,0.158375f,-0.987379f},{0.997057f,0.0292024f,-0.0708805f},{1.69018e-007f,-0.854123f,-0.520071f},
{5.88791e-008f,-0.595084f,-0.803663f},{4.9126e-008f,-0.248255f,-0.968695f},{1.0f,1.07203e-007f,3.29935e-007f},
{1.0f,1.07203e-007f,7.78878e-008f},{1.0f,8.18961e-008f,0.0f},{1.0f,5.36018e-008f,-3.89439e-008f},
{1.0f,2.04741e-008f,-6.30126e-008f},{1.0f,-2.53075e-008f,-7.78879e-008f},{1.0f,-1.07204e-007f,-7.78879e-008f},
{1.0f,-3.46918e-007f,-1.41482e-013f},{-1.0f,1.07203e-007f,3.29935e-007f},{-1.0f,1.07203e-007f,7.78878e-008f},
{-1.0f,8.18961e-008f,0.0f},{-1.0f,5.36018e-008f,-3.89439e-008f},{-1.0f,2.04741e-008f,-6.30126e-008f},
{-1.0f,-2.53075e-008f,-7.78879e-008f},{-1.0f,-3.46918e-007f,-1.41482e-013f},{-1.0f,-1.07204e-007f,-7.78879e-008f},
{1.0f,1.07203e-007f,3.29935e-007f},{1.0f,1.07203e-007f,7.78877e-008f},{1.0f,5.36018e-008f,-3.89439e-008f},
{1.0f,2.04741e-008f,-6.30126e-008f},{1.0f,-1.07203e-007f,-7.78879e-008f},{1.0f,-3.46918e-007f,0.0f},
{-1.0f,1.07203e-007f,3.29935e-007f},{-1.0f,1.07203e-007f,7.78877e-008f},{-1.0f,5.36018e-008f,-3.89439e-008f},
{-1.0f,2.04741e-008f,-6.30126e-008f},{-1.0f,-3.46918e-007f,0.0f},{-1.0f,-1.07203e-007f,-7.78879e-008f},
{-0.312209f,-0.498156f,0.808929f},{0.0f,-0.508499f,0.861062f},{-0.132915f,-0.455331f,0.880345f},
{-0.100149f,-0.494396f,0.863448f},{-0.14973f,-0.539738f,0.82841f},{-0.166085f,-0.511804f,0.842895f},
{-0.46679f,-0.492984f,0.734216f},{-0.307678f,-0.460976f,0.832367f},{-0.497936f,-0.457317f,0.736832f},
{-0.493442f,-0.5276f,0.691487f},{0.312209f,-0.498156f,0.808929f},{0.100149f,-0.494396f,0.863448f},
{0.132915f,-0.455331f,0.880345f},{0.166085f,-0.511804f,0.842895f},{0.14973f,-0.539738f,0.82841f},
{0.307678f,-0.460976f,0.832367f},{0.46679f,-0.492984f,0.734216f},{0.497936f,-0.457317f,0.736832f},
{0.493442f,-0.5276f,0.691487f},{0.524041f,-0.675253f,0.519052f},{-0.456172f,-0.751499f,0.476609f},
{0.205949f,-0.961992f,-0.179321f},{0.504124f,-0.687561f,-0.522608f},{0.869385f,-0.465668f,-0.1653f},
{-0.633883f,-0.674041f,0.37929f},{-0.88049f,-0.277773f,0.38416f},{-0.284101f,-0.412601f,0.865475f},
{0.549885f,-0.406711f,0.729529f},{0.937095f,-0.342021f,0.0698194f},{0.608228f,-0.291667f,-0.738234f},
{0.517871f,-0.32311f,-0.792092f},{-0.0666881f,0.997392f,0.0276175f},{-0.0679401f,0.997436f,0.0224902f},
{-0.0654343f,0.99732f,0.032744f},{-0.32536f,-0.921415f,0.212452f},{-0.304708f,-0.929069f,0.209725f},
{0.791056f,-0.239694f,-0.56283f},{0.799282f,-0.24606f,-0.548272f},{-0.769459f,-0.267238f,0.580101f},
{-0.774634f,-0.270741f,0.571526f},{-0.205949f,-0.961992f,-0.179321f},{0.456172f,-0.751499f,0.476609f},
{-0.524041f,-0.675253f,0.519052f},{-0.504124f,-0.687561f,-0.522608f},{-0.869385f,-0.465668f,-0.1653f},
{0.633883f,-0.674041f,0.37929f},{0.88049f,-0.277773f,0.38416f},{0.284101f,-0.412601f,0.865475f},
{-0.549885f,-0.406711f,0.729529f},{-0.937095f,-0.342021f,0.0698194f},{-0.608228f,-0.291667f,-0.738234f},
{-0.517871f,-0.32311f,-0.792092f},{0.0666881f,0.997392f,0.0276175f},{0.0654343f,0.99732f,0.032744f},
{0.0679401f,0.997436f,0.0224902f},{0.304708f,-0.929069f,0.209725f},{0.32536f,-0.921415f,0.212452f},
{-0.791056f,-0.239694f,-0.56283f},{-0.799282f,-0.24606f,-0.548272f},{0.774634f,-0.270741f,0.571526f},
{0.769459f,-0.267238f,0.580101f},{-0.208913f,0.977926f,-0.00407805f},{-0.313271f,0.949583f,-0.0123751f},
{-0.307772f,0.951441f,-0.00600781f},{-0.0967574f,0.995296f,-0.00494027f},{0.208913f,0.977926f,-0.00407805f},
{0.313271f,0.949583f,-0.0123751f},{0.307772f,0.951441f,-0.00600781f},{0.0967574f,0.995296f,-0.00494027f}
};
static GLfloat textures [822][2] = {
{0.298301f,0.0429086f},{0.298301f,0.0263816f},{0.371478f,0.0223928f},
{0.374629f,0.0429086f},{0.298301f,0.00817141f},{0.367588f,0.00817141f},
{0.197447f,0.0862799f},{0.298301f,0.0862799f},{0.298301f,0.107596f},
{0.197447f,0.108737f},{0.377182f,0.0862799f},{0.371685f,0.105515f},
{0.298301f,0.137472f},{0.197447f,0.141467f},{0.197447f,0.145593f},
{0.298301f,0.14169f},{0.197447f,0.00817141f},{0.197447f,0.0277744f},
{0.197447f,0.0531808f},{0.298301f,0.0531808f},{0.197447f,0.0429086f},
{0.34569f,0.132538f},{0.34001f,0.136677f},{0.376314f,0.0531807f},
{0.0965935f,0.0429086f},{0.0202656f,0.0429086f},{0.0234166f,0.0223928f},
{0.0965935f,0.0263816f},{0.0273061f,0.00817141f},{0.0965935f,0.00817141f},
{0.0965935f,0.107596f},{0.0965935f,0.0862799f},{0.0232094f,0.105515f},
{0.0177121f,0.0862799f},{0.0965935f,0.137472f},{0.0965935f,0.14169f},
{0.0965935f,0.0531808f},{0.0492047f,0.132538f},{0.0548843f,0.136677f},
{0.01858f,0.0531808f},{0.0647867f,0.904423f},{0.181848f,0.904423f},
{0.181848f,0.943731f},{0.0647867f,0.943731f},{0.181848f,0.980464f},
{0.0647867f,0.980464f},{0.181848f,0.997585f},{0.0647867f,0.997585f},
{0.000980679f,0.943731f},{0.00103989f,0.902808f},{0.000938993f,0.980464f},
{0.000885657f,0.997585f},{0.298909f,0.904423f},{0.298909f,0.943731f},
{0.298909f,0.980464f},{0.298909f,0.997584f},{0.362656f,0.902808f},
{0.362715f,0.943731f},{0.362756f,0.980464f},{0.36281f,0.997584f},
{0.550561f,0.896952f},{0.550561f,0.886849f},{0.467174f,0.892884f},
{0.452944f,0.88736f},{0.467888f,0.886202f},{0.439794f,0.881055f},
{0.449058f,0.892872f},{0.454342f,0.889591f},{0.246854f,0.797143f},
{0.340107f,0.840172f},{0.329254f,0.791458f},{0.728683f,0.877409f},
{0.79168f,0.895023f},{0.795671f,0.877842f},{0.798515f,0.861952f},
{0.419784f,0.880681f},{0.246854f,0.881002f},{0.550561f,0.871039f},
{0.433999f,0.863934f},{0.719508f,0.894613f},{0.692161f,0.930028f},
{0.660743f,0.964329f},{0.842727f,0.929237f},{0.647534f,0.978749f},
{0.550561f,0.965106f},{0.550561f,0.979666f},{0.505275f,0.962836f},
{0.707041f,0.909218f},{0.727791f,0.961832f},{0.550561f,0.930111f},
{0.476896f,0.929152f},{0.930732f,0.8908f},{0.892948f,0.918128f},
{0.945378f,0.917823f},{0.930732f,0.8908f},{0.939721f,0.871565f},
{0.87431f,0.878352f},{0.89787f,0.796423f},{0.959551f,0.807678f},
{0.902798f,0.766568f},{0.826559f,0.828309f},{0.889871f,0.828194f},
{0.83826f,0.791299f},{0.840442f,0.752867f},{0.960481f,0.828194f},
{0.882084f,0.857451f},{0.939721f,0.871565f},{0.960481f,0.828194f},
{0.960481f,0.828194f},{0.940709f,0.838466f},{0.857815f,0.910554f},
{0.930732f,0.8908f},{0.865356f,0.895833f},{0.246854f,0.86944f},
{0.118524f,0.866984f},{0.930732f,0.8908f},{0.384368f,0.865344f},
{0.550561f,0.842467f},{0.43143f,0.835614f},{0.246854f,0.84826f},
{0.550561f,0.791516f},{0.43143f,0.785777f},{0.246854f,0.720337f},
{0.43143f,0.720613f},{0.550561f,0.720796f},{0.638951f,0.799575f},
{0.642679f,0.721276f},{0.178835f,0.851591f},{0.115608f,0.877766f},
{0.246854f,0.890397f},{0.211235f,0.7986f},{0.215968f,0.720267f},
{0.930732f,0.8908f},{0.0327669f,0.803084f},{0.0319546f,0.763935f},
{0.00660626f,0.763935f},{0.00294988f,0.803084f},{0.00660626f,0.763935f},
{0.00660626f,0.763935f},{0.0279236f,0.723387f},{3.97295e-005f,0.723012f},
{0.00294988f,0.803084f},{0.00388258f,0.820204f},{0.00388258f,0.820204f},
{0.0380561f,0.820204f},{0.942823f,0.921962f},{0.945378f,0.917823f},
{0.00388258f,0.820204f},{0.0219652f,0.84126f},{0.0474358f,0.838126f},
{0.0579433f,0.863729f},{0.0689196f,0.855396f},{0.663117f,0.853112f},
{0.959551f,0.807678f},{0.958709f,0.793457f},{0.939721f,0.871565f},
{0.940709f,0.838466f},{0.550561f,0.886849f},{0.550561f,0.896952f},
{0.467174f,0.892884f},{0.467888f,0.886202f},{0.452944f,0.88736f},
{0.439794f,0.881055f},{0.449058f,0.892872f},{0.454342f,0.889592f},
{0.340107f,0.840172f},{0.246854f,0.797143f},{0.329254f,0.791458f},
{0.79168f,0.895023f},{0.728683f,0.877409f},{0.795671f,0.877843f},
{0.798515f,0.861952f},{0.419784f,0.880682f},{0.246854f,0.881002f},
{0.550561f,0.871039f},{0.433999f,0.863934f},{0.719508f,0.894613f},
{0.660742f,0.964329f},{0.692161f,0.930028f},{0.842727f,0.929237f},
{0.647534f,0.978749f},{0.550561f,0.979666f},{0.550561f,0.965106f},
{0.505275f,0.962836f},{0.707041f,0.909218f},{0.727791f,0.961832f},
{0.550561f,0.930111f},{0.476896f,0.929152f},{0.892948f,0.918128f},
{0.930732f,0.8908f},{0.945378f,0.917824f},{0.939721f,0.871565f},
{0.930732f,0.8908f},{0.87431f,0.878352f},{0.959551f,0.807678f},
{0.902798f,0.766568f},{0.889871f,0.828194f},{0.826559f,0.828309f},
{0.83826f,0.7913f},{0.840442f,0.752867f},{0.960481f,0.828194f},
{0.882084f,0.857451f},{0.939721f,0.871565f},{0.960481f,0.828194f},
{0.960481f,0.828194f},{0.940709f,0.838466f},{0.930732f,0.8908f},
{0.857815f,0.910554f},{0.865356f,0.895833f},{0.246854f,0.86944f},
{0.118524f,0.866984f},{0.930732f,0.8908f},{0.384368f,0.865344f},
{0.550561f,0.842467f},{0.43143f,0.835614f},{0.246854f,0.84826f},
{0.550561f,0.791516f},{0.43143f,0.785777f},{0.246854f,0.720337f},
{0.43143f,0.720613f},{0.550561f,0.720796f},{0.642679f,0.721276f},
{0.638951f,0.799575f},{0.178835f,0.851591f},{0.246854f,0.890397f},
{0.115608f,0.877766f},{0.211235f,0.7986f},{0.215968f,0.720267f},
{0.930732f,0.8908f},{0.0327669f,0.803084f},{0.00294989f,0.803084f},
{0.00660627f,0.763935f},{0.0319546f,0.763935f},{0.00660627f,0.763935f},
{3.97405e-005f,0.723012f},{0.0279236f,0.723387f},{0.00660627f,0.763935f},
{0.00388259f,0.820204f},{0.00294989f,0.803084f},{0.0380561f,0.820204f},
{0.00388259f,0.820204f},{0.942823f,0.921962f},{0.945378f,0.917824f},
{0.0474358f,0.838126f},{0.0219652f,0.84126f},{0.00388259f,0.820204f},
{0.0689196f,0.855396f},{0.0579433f,0.863729f},{0.663117f,0.853112f},
{0.959551f,0.807678f},{0.958709f,0.793457f},{0.939721f,0.871565f},
{0.940709f,0.838466f},{0.386875f,0.516557f},{0.414107f,0.614527f},
{0.289951f,0.628056f},{0.289951f,0.516557f},{0.156512f,0.632137f},
{0.154909f,0.516557f},{0.289951f,0.700596f},{0.289951f,0.700596f},
{0.158706f,0.703615f},{0.492156f,0.689245f},{0.289951f,0.700596f},
{0.994882f,0.616996f},{0.938704f,0.616996f},{0.940384f,0.516557f},
{1.0f,0.516557f},{0.994882f,0.616996f},{0.89347f,0.616995f},
{0.896631f,0.516557f},{0.687747f,0.6061f},{0.68608f,0.516557f},
{0.773501f,0.516557f},{0.772272f,0.60948f},{0.593658f,0.60081f},
{0.54234f,0.605738f},{0.5312f,0.516557f},{0.593658f,0.516557f},
{0.0922393f,0.632647f},{0.0863703f,0.516557f},{0.0173332f,0.633136f},
{0.0f,0.516557f},{0.031489f,0.516557f},{0.0458246f,0.632933f},
{0.0173332f,0.633136f},{0.994882f,0.616996f},{0.98592f,0.658533f},
{0.936046f,0.666523f},{0.936046f,0.666523f},{0.885824f,0.666272f},
{0.936046f,0.666523f},{0.593658f,0.632832f},{0.593658f,0.632832f},
{0.548372f,0.644912f},{0.690632f,0.633831f},{0.593658f,0.632832f},
{0.0650627f,0.699359f},{0.0469801f,0.696774f},{0.0173332f,0.633136f},
{0.0469801f,0.696774f},{0.0650627f,0.699359f},{0.101041f,0.702663f},
{0.0650627f,0.699359f},{0.690632f,0.633831f},{0.770889f,0.645937f},
{0.690632f,0.633831f},{0.885824f,0.666272f},{0.770889f,0.645937f},
{0.770889f,0.645937f},{0.101041f,0.702663f},{0.158706f,0.703615f},
{0.101041f,0.702663f},{0.289951f,0.405058f},{0.414106f,0.418587f},
{0.156512f,0.400976f},{0.289951f,0.332518f},{0.158706f,0.329498f},
{0.289951f,0.332518f},{0.289951f,0.332518f},{0.492156f,0.343868f},
{0.289951f,0.332518f},{0.994882f,0.416118f},{0.938704f,0.416118f},
{0.994882f,0.416118f},{0.89347f,0.416118f},{0.687747f,0.427013f},
{0.772272f,0.423633f},{0.593658f,0.432303f},{0.54234f,0.427375f},
{0.0922393f,0.400466f},{0.0f,0.516557f},{0.0173332f,0.399977f},
{0.0173332f,0.399977f},{0.0458247f,0.400181f},{0.936046f,0.366591f},
{0.98592f,0.37458f},{0.994882f,0.416118f},{0.98592f,0.37458f},
{0.936046f,0.366591f},{0.885824f,0.366841f},{0.936046f,0.366591f},
{0.593658f,0.400281f},{0.548372f,0.388202f},{0.593658f,0.400281f},
{0.593658f,0.400281f},{0.690631f,0.399282f},{0.593658f,0.400281f},
{0.0173332f,0.399977f},{0.0469801f,0.336339f},{0.0650627f,0.333754f},
{0.0650627f,0.333754f},{0.0650627f,0.333754f},{0.101041f,0.33045f},
{0.690631f,0.399282f},{0.770889f,0.387177f},{0.770889f,0.387177f},
{0.885824f,0.366841f},{0.770889f,0.387177f},{0.101041f,0.33045f},
{0.158706f,0.329498f},{0.101041f,0.33045f},{0.512613f,0.101791f},
{0.512613f,0.16456f},{0.425591f,0.16456f},{0.425591f,0.101791f},
{0.512613f,0.0390227f},{0.425591f,0.0390227f},{0.512613f,0.000229538f},
{0.425591f,0.000229538f},{0.425591f,0.000229451f},{0.425591f,0.0390227f},
{0.512613f,0.101791f},{0.425591f,0.101791f},{0.512613f,0.16456f},
{0.425591f,0.16456f},{0.512613f,0.203353f},{0.425591f,0.203353f},
{0.512378f,0.16456f},{0.425355f,0.16456f},{0.425355f,0.101791f},
{0.512378f,0.101791f},{0.425355f,0.0390227f},{0.512378f,0.0390227f},
{0.425355f,0.000229538f},{0.512378f,0.000229538f},{0.512378f,0.000229538f},
{0.512378f,0.0390227f},{0.512378f,0.101791f},{0.425355f,0.16456f},
{0.512378f,0.16456f},{0.425355f,0.203353f},{0.512378f,0.203353f},
{0.512378f,0.203353f},{0.512677f,0.101843f},{0.512677f,0.164611f},
{0.425655f,0.164611f},{0.425655f,0.101843f},{0.512677f,0.0390739f},
{0.425655f,0.0390739f},{0.512677f,0.000280653f},{0.425655f,0.000280653f},
{0.425655f,0.000280653f},{0.425655f,0.0390739f},{0.512677f,0.101843f},
{0.425655f,0.101843f},{0.512677f,0.203405f},{0.425655f,0.203405f},
{0.51245f,0.164425f},{0.425428f,0.164425f},{0.425428f,0.101656f},
{0.51245f,0.101656f},{0.425428f,0.0388876f},{0.51245f,0.0388876f},
{0.425428f,9.44798e-005f},{0.51245f,9.44798e-005f},{0.51245f,0.0388876f},
{0.425428f,0.164425f},{0.425428f,0.203218f},{0.51245f,0.203218f},
{0.200788f,0.185021f},{0.301642f,0.185021f},{0.301642f,0.241762f},
{0.200788f,0.185021f},{0.0999345f,0.241762f},{0.0999345f,0.185021f},
{0.200788f,0.185021f},{0.379964f,0.239346f},{0.306829f,0.304727f},
{0.0947477f,0.304727f},{0.0365678f,0.185021f},{0.0999345f,0.185021f},
{0.0216119f,0.241762f},{0.0365678f,0.185021f},{0.00791303f,0.304727f},
{0.0216119f,0.241762f},{0.365009f,0.182605f},{0.379964f,0.239346f},
{0.301642f,0.185021f},{0.365009f,0.182605f},{0.0216119f,0.241762f},
{0.393663f,0.302311f},{0.379964f,0.239346f},{0.901218f,0.153664f},
{0.901219f,0.204217f},{0.855607f,0.203586f},{0.855607f,0.00481351f},
{0.901218f,0.00418262f},{0.901218f,0.0547356f},{0.596824f,0.156489f},
{0.596824f,0.202596f},{0.583018f,0.193358f},{0.583018f,0.0150413f},
{0.596824f,0.0058028f},{0.596824f,0.0519099f},{0.705227f,0.203263f},
{0.689934f,0.203214f},{0.689934f,0.153664f},{0.689934f,0.0547356f},
{0.689934f,0.00518524f},{0.705227f,0.00513604f},{0.568662f,0.161924f},
{0.560557f,0.1042f},{0.568662f,0.0464755f},{0.568662f,0.0464755f},
{0.583018f,0.0150413f},{0.583018f,0.193358f},{0.583018f,0.193358f},
{0.568662f,0.161924f},{0.583018f,0.0150413f},{0.705227f,0.00513604f},
{0.705227f,0.203263f},{0.705227f,0.00513604f},{0.796619f,0.00776763f},
{0.796619f,0.200632f},{0.796619f,0.200632f},{0.796619f,0.00776763f},
{0.855607f,0.00481351f},{0.855607f,0.203586f},{0.855607f,0.203586f},
{0.855607f,0.00481351f},{0.999141f,0.05191f},{0.899373f,0.0519099f},
{0.911339f,0.0519099f},{0.943803f,0.0519099f},{0.978381f,0.05191f},
{0.992267f,0.0519099f},{0.99806f,0.05191f},{0.999141f,0.00617123f},
{0.999141f,0.05191f},{0.99806f,0.00322764f},{0.999141f,0.00617123f},
{0.99806f,0.00322764f},{0.99806f,0.00322764f},{0.992267f,0.00271222f},
{0.68759f,0.0519099f},{0.671548f,0.0519099f},{0.641685f,0.0519099f},
{0.617123f,0.0519099f},{0.601841f,0.0519099f},{0.606486f,0.0519099f},
{0.599222f,0.0519099f},{0.59882f,0.0519099f},{0.596824f,0.0058028f},
{0.59882f,0.00538082f},{0.59882f,0.00538082f},{0.599222f,0.00512629f},
{0.59882f,0.00538082f},{0.601841f,0.00478991f},{0.606486f,0.004137f},
{0.606486f,0.004137f},{0.617123f,0.00327989f},{0.606486f,0.004137f},
{0.599222f,0.00512629f},{0.601841f,0.00478991f},{0.599222f,0.00512629f},
{0.992267f,0.00271222f},{0.978381f,0.00179007f},{0.992267f,0.00271222f},
{0.943803f,0.000278389f},{0.978381f,0.00179007f},{0.978381f,0.00179007f},
{0.901218f,0.00418262f},{0.899373f,0.0f},{0.901218f,0.00418262f},
{0.911338f,0.000210642f},{0.899373f,0.0f},{0.911338f,0.000210642f},
{0.943803f,0.000278389f},{0.943803f,0.000278389f},{0.689934f,0.00518524f},
{0.68759f,0.00130844f},{0.68759f,0.00130844f},{0.671548f,0.00127125f},
{0.68759f,0.00130844f},{0.641685f,0.00195421f},{0.671548f,0.00127125f},
{0.641685f,0.00195421f},{0.617123f,0.00327989f},{0.617123f,0.00327989f},
{0.899373f,0.156489f},{0.999141f,0.156489f},{0.911339f,0.156489f},
{0.943803f,0.156489f},{0.992267f,0.156489f},{0.978381f,0.156489f},
{0.99806f,0.156489f},{0.999141f,0.202228f},{0.99806f,0.205172f},
{0.99806f,0.205172f},{0.992267f,0.205687f},{0.99806f,0.205172f},
{0.999141f,0.05191f},{0.999141f,0.156489f},{0.999141f,0.05191f},
{0.68759f,0.156489f},{0.671548f,0.156489f},{0.641685f,0.156489f},
{0.617123f,0.156489f},{0.606486f,0.156489f},{0.601841f,0.156489f},
{0.599222f,0.156489f},{0.59882f,0.156489f},{0.59882f,0.203018f},
{0.596824f,0.202596f},{0.596824f,0.202596f},{0.59882f,0.203018f},
{0.599222f,0.203273f},{0.59882f,0.203018f},{0.606486f,0.204262f},
{0.601841f,0.203609f},{0.601841f,0.203609f},{0.606486f,0.204262f},
{0.617123f,0.205119f},{0.606486f,0.204262f},{0.599222f,0.203273f},
{0.601841f,0.203609f},{0.599222f,0.203273f},{0.992267f,0.205687f},
{0.978381f,0.206609f},{0.978381f,0.206609f},{0.943803f,0.208121f},
{0.978381f,0.206609f},{0.899373f,0.208399f},{0.901219f,0.204217f},
{0.901219f,0.204217f},{0.911339f,0.208189f},{0.899373f,0.208399f},
{0.911339f,0.208189f},{0.943803f,0.208121f},{0.943803f,0.208121f},
{0.689934f,0.203214f},{0.68759f,0.207091f},{0.68759f,0.207091f},
{0.671548f,0.207128f},{0.671548f,0.207128f},{0.641685f,0.206445f},
{0.641685f,0.206445f},{0.617123f,0.205119f},{0.617123f,0.205119f},
{0.797595f,0.666571f},{0.741201f,0.648323f},{0.684808f,0.666571f},
{0.797595f,0.666571f},{0.684808f,0.666571f},{0.649955f,0.714345f},
{0.797595f,0.666571f},{0.649955f,0.714345f},{0.649955f,0.773397f},
{0.649955f,0.773397f},{0.684808f,0.82117f},{0.797595f,0.666571f},
{0.684808f,0.82117f},{0.741201f,0.839418f},{0.797595f,0.82117f},
{0.832448f,0.773397f},{0.832448f,0.714345f},{0.797595f,0.666571f},
{0.832448f,0.773397f},{0.684808f,0.666571f},{0.741201f,0.648323f},
{0.797595f,0.666571f},{0.649955f,0.714345f},{0.797595f,0.666571f},
{0.649955f,0.773397f},{0.797595f,0.666571f},{0.684808f,0.82117f},
{0.649955f,0.773397f},{0.797595f,0.666571f},{0.741201f,0.839418f},
{0.684808f,0.82117f},{0.797595f,0.666571f},{0.797595f,0.82117f},
{0.797595f,0.666571f},{0.797595f,0.82117f},{0.832448f,0.714345f},
{0.832448f,0.773397f},{0.684808f,0.666571f},{0.741201f,0.648323f},
{0.649955f,0.714345f},{0.684808f,0.666571f},{0.649955f,0.773397f},
{0.649955f,0.714345f},{0.684808f,0.82117f},{0.741201f,0.839419f},
{0.684808f,0.82117f},{0.797595f,0.82117f},{0.741201f,0.839419f},
{0.797595f,0.666571f},{0.797595f,0.82117f},{0.832448f,0.773397f},
{0.797594f,0.666571f},{0.741201f,0.648323f},{0.684808f,0.666571f},
{0.797594f,0.666571f},{0.684808f,0.666571f},{0.649955f,0.714345f},
{0.797594f,0.666571f},{0.649955f,0.714345f},{0.649955f,0.773397f},
{0.649955f,0.773397f},{0.684808f,0.82117f},{0.684808f,0.82117f},
{0.741201f,0.839419f},{0.797594f,0.666571f},{0.832448f,0.714345f},
{0.797594f,0.666571f},{0.797594f,0.666571f},{0.797927f,0.666444f},
{0.741534f,0.648196f},{0.68514f,0.666444f},{0.797927f,0.666444f},
{0.68514f,0.666444f},{0.650287f,0.714218f},{0.797927f,0.666444f},
{0.650287f,0.714218f},{0.650287f,0.773269f},{0.650287f,0.773269f},
{0.68514f,0.821043f},{0.68514f,0.821043f},{0.741534f,0.839291f},
{0.797927f,0.821043f},{0.83278f,0.773269f},{0.83278f,0.714218f},
{0.797927f,0.666444f},{0.83278f,0.773269f},{0.68514f,0.666444f},
{0.741534f,0.648196f},{0.650287f,0.714218f},{0.68514f,0.666444f},
{0.650287f,0.773269f},{0.68514f,0.821043f},{0.741534f,0.839291f},
{0.797927f,0.821043f},{0.797927f,0.666444f},{0.83278f,0.773269f},
{0.797927f,0.821043f},{0.83278f,0.714218f},{0.83278f,0.773269f},
{0.68514f,0.666444f},{0.741534f,0.648196f},{0.797927f,0.666444f},
{0.68514f,0.666444f},{0.650287f,0.714218f},{0.797927f,0.666444f},
{0.68514f,0.821043f},{0.650287f,0.773269f},{0.797927f,0.666444f},
{0.741534f,0.839291f},{0.68514f,0.821043f},{0.797927f,0.666444f},
{0.797927f,0.821043f},{0.741534f,0.839291f},{0.797927f,0.666444f},
{0.797927f,0.821043f},{0.797927f,0.666444f},{0.797927f,0.666444f},
{0.741534f,0.648196f},{0.68514f,0.666444f},{0.68514f,0.821043f},
{0.68514f,0.821043f},{0.797927f,0.821043f},{0.83278f,0.773269f},
{0.83278f,0.714218f},{0.797927f,0.666444f},{0.739649f,0.30046f},
{0.644841f,0.317208f},{0.644841f,0.226484f},{0.644841f,0.226484f},
{0.73439f,0.218639f},{0.743215f,0.368501f},{0.644841f,0.39098f},
{0.743215f,0.368501f},{0.773726f,0.218681f},{0.796956f,0.259592f},
{0.73439f,0.218639f},{0.773726f,0.218681f},{0.818242f,0.301266f},
{0.743215f,0.368501f},{0.796956f,0.259592f},{0.818242f,0.301266f},
{0.550032f,0.30046f},{0.555291f,0.218639f},{0.644841f,0.39098f},
{0.546466f,0.368501f},{0.546466f,0.368501f},{0.515955f,0.218681f},
{0.555291f,0.218639f},{0.492726f,0.259592f},{0.515955f,0.218681f},
{0.546466f,0.368501f},{0.471439f,0.301266f},{0.471439f,0.301266f},
{0.492726f,0.259592f},{0.445969f,0.499232f},{0.490765f,0.499232f},
{0.493103f,0.462202f},{0.441668f,0.463996f},{0.432028f,0.472927f},
{0.503452f,0.457728f},{0.508927f,0.462018f},{0.495715f,0.508276f},
{0.439911f,0.509354f},{0.423042f,0.472927f},{0.438828f,0.454099f},
{0.493662f,0.449306f},{0.498929f,0.441744f},{0.514551f,0.454373f},
{0.514551f,0.454373f},{0.510839f,0.44815f},{0.506117f,0.443932f},
{0.506117f,0.443932f},{0.510839f,0.44815f},{0.514551f,0.454373f},
{0.493033f,0.462153f},{0.490695f,0.499184f},{0.445899f,0.499184f},
{0.441598f,0.463947f},{0.431958f,0.472879f},{0.503383f,0.457679f},
{0.508858f,0.46197f},{0.495646f,0.508227f},{0.439842f,0.509306f},
{0.422973f,0.472878f},{0.438759f,0.454051f},{0.493593f,0.449257f},
{0.514481f,0.454324f},{0.498859f,0.441695f},{0.514481f,0.454324f},
{0.506047f,0.443884f},{0.510769f,0.448101f},{0.506047f,0.443884f},
{0.506047f,0.443884f},{0.498859f,0.441695f},{0.506047f,0.443884f},
{0.514481f,0.454324f},{0.510769f,0.448101f},{0.514481f,0.454324f},
{0.443935f,0.617859f},{0.426768f,0.581432f},{0.442894f,0.562604f},
{0.442894f,0.562604f},{0.500608f,0.557811f},{0.502556f,0.616781f},
{0.502556f,0.616781f},{0.443935f,0.617859f},{0.442894f,0.562604f},
{0.515673f,0.570523f},{0.502556f,0.616781f},{0.500608f,0.557811f},
{0.426942f,0.5811f},{0.444109f,0.617527f},{0.443067f,0.562272f},
{0.502729f,0.616449f},{0.500781f,0.557479f},{0.443067f,0.562272f},
{0.443067f,0.562272f},{0.444109f,0.617527f},{0.502729f,0.616449f},
{0.502729f,0.616449f},{0.515846f,0.570191f},{0.500781f,0.557479f}
};
/*Material indicies*/
/*{material index,face count}*/
static int material_ref [1][2] = {
{0,732}
};




struct DIB2D{
 BITMAPINFOHEADER *Info;
 RGBQUAD *palette;
 BYTE    *bits;
};
struct GLTXTLOAD{
 GLint format;
 GLint perpixel;
 GLint Width;
 GLint Height;
 BYTE* bits;
};


#define VERTEX_BUFFER 0
#define NORMAL_BUFFER 1
#define TEXTURE_BUFFER 2


GLuint buffers[3];

void SetArrays(GLfixed vertexArray[][3], GLfixed normalArray[][3], GLfixed textureArray[][2]) {
	int index = 0;

	for(int i=sizeof(face_indicies)/sizeof(face_indicies[0]);i--;)
	{
#if 0
		if(!mcount)
		{
			SelectMaterial(material_ref[mindex][0]);
			mcount=material_ref[mindex][1];
			mindex++;
		}
		mcount--;
#endif
		for(int j=0;j<3;j++)
		{
			int vi=face_indicies[i][j];
			int ni=face_indicies[i][j+3];//Normal index
			int ti=face_indicies[i][j+6];//Texture index

			normalArray[index][0] = EGL_FixedFromFloat(normals[ni][0]);
			normalArray[index][1] = EGL_FixedFromFloat(normals[ni][1]);
			normalArray[index][2] = EGL_FixedFromFloat(normals[ni][2]);
			//glNormal3f (normals[ni][0],normals[ni][1],normals[ni][2]);
			textureArray[index][0] = EGL_FixedFromFloat(textures[ti][0]);
			textureArray[index][1] = EGL_FixedFromFloat(textures[ti][1]);
			//glTexCoord2f(textures[ti][0],textures[ti][1]);
			vertexArray[index][0] = EGL_FixedFromFloat(vertices[vi][0]);
			vertexArray[index][1] = EGL_FixedFromFloat(vertices[vi][1]);
			vertexArray[index][2] = EGL_FixedFromFloat(vertices[vi][2]);
			//glVertex3f (vertices[vi][0],vertices[vi][1],vertices[vi][2]);

			index++;
		}
	}

#if USE_BUFFERS

	glGenBuffers(3, buffers);
	glBindBuffer(GL_ARRAY_BUFFER, buffers[VERTEX_BUFFER]);
	glBufferData(GL_ARRAY_BUFFER, index * sizeof(GLfixed) * 3, vertexArray, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, buffers[NORMAL_BUFFER]);
	glBufferData(GL_ARRAY_BUFFER, index * sizeof(GLfixed) * 3, normalArray, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, buffers[TEXTURE_BUFFER]);
	glBufferData(GL_ARRAY_BUFFER, index * sizeof(GLfixed) * 2, textureArray, GL_STATIC_DRAW);

#endif
}


// --------- END OF INCLUDED SNIPPET

float angle = 30.0f;
float x = 0, minX = -100, maxX = 100;
float y = 0, minY = -150, maxY = 150;
float z = 0, minZ = -200, maxZ = 200;

#define SIZE (3 * sizeof(face_indicies)/sizeof(face_indicies[0]))

GLfixed egl_vertices[SIZE][3];
GLfixed egl_normals[SIZE][3];
GLfixed egl_textures[SIZE][2];


extern "C" BOOL InitOpenGL(HWND hWnd) {

	HIDPI_InitScaling();


	// Initialize OGL
    HDC hdc = GetWindowDC(hWnd);

	//g_display = eglGetDisplay(hdc);
	g_display = eglGetDisplay(0);

	EGLint major, minor;

	if (!eglInitialize(g_display, &major, &minor)) {
		return FALSE;
	}

    EGLConfig configs[10];
	EGLint matchingConfigs;
	EGLint attribList[] = {EGL_NONE };	// extend this

	if (!eglChooseConfig(g_display, attribList, &configs[0], 10, &matchingConfigs)) {
		return FALSE;
	}

	if (matchingConfigs < 1) {
		return FALSE;
	}

	EGLConfig config = configs[0];	// pick any

	g_surface = eglCreateWindowSurface(g_display, config, hWnd, attribList);

	// test for error

	g_context = eglCreateContext(g_display, config, 0, attribList);
	eglMakeCurrent(g_display, g_surface, g_surface, g_context);

	// TO DO: test for error
	bool success = LoadTexture(L"\\dodge.jpg");

	if (!success) {
		MessageBoxW(hWnd, L"Cannot load texture image. Terminating application", L"Error", MB_OK);
		return FALSE;
	}

	// Initialize the car data
	SetArrays(egl_vertices, egl_normals, egl_textures);

	return TRUE;
}


extern "C" void PaintProc(HWND hWnd) {
	// the actual painting code will go here
	HDC hdc;
	PAINTSTRUCT ps;
	DWORD startTicks = GetTickCount();

	// rendering preamble
	RECT rt;
	hdc = BeginPaint(hWnd, &ps);
	//EGLDisplay display = eglGetDisplay(hdc);
	EGLDisplay display = eglGetDisplay(0);

	GetClientRect(hWnd, &rt);

	eglMakeCurrent(display, g_surface, g_surface, g_context);


	glViewport(rt.left, rt.top, rt.right - rt.left, rt.bottom - rt.top);
	glDepthRangex(EGL_FixedFromInt(0), EGL_FixedFromInt(1));
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustumx(EGL_FixedFromInt((rt.left - rt.right) / 2),
		EGL_FixedFromInt((rt.right - rt.left) / 2),
		EGL_FixedFromInt((rt.top - rt.bottom) / 2),
		EGL_FixedFromInt((rt.bottom - rt.top) / 2),
		EGL_FixedFromInt(100),
		EGL_FixedFromInt(1000));
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatex(EGL_FixedFromFloat(x), EGL_FixedFromFloat(y), EGL_FixedFromFloat(x - 300));
	glScalex(EGL_FixedFromInt(220), EGL_FixedFromInt(220), EGL_FixedFromInt(220));
	glRotatex(EGL_FixedFromFloat(angle), 0, EGL_FixedFromInt(1), 0);
	glRotatex(EGL_FixedFromFloat(280.0f), EGL_FixedFromInt(1), 0, 0);

	// actual GL rendering goes here
	glClearColorx(0, 0, 0, 0);		// black background
	glClearDepthx(EGL_ONE);
	//glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	DWORD clearTicks = GetTickCount();

	//glShadeModel(GL_SMOOTH);
	glShadeModel(GL_FLAT);
	//glEnable(GL_LIGHTING);
	glDisable(GL_LIGHTING);
	glEnable(GL_COLOR_MATERIAL);
	GLfixed materialColor[] = { EGL_ONE, EGL_ONE, EGL_ONE, 0 };
	glColor4x(EGL_ONE, EGL_ONE, EGL_ONE, EGL_ONE);
	glMaterialxv(GL_FRONT, GL_DIFFUSE, materialColor);
	glMaterialxv(GL_BACK, GL_DIFFUSE, materialColor);
	glLightxv(0, GL_DIFFUSE, materialColor);
	//glEnable(GL_RESCALE_NORMAL);
	glEnable(GL_DEPTH_TEST);
	//glDisable(GL_DEPTH_TEST);
	glEnable(GL_LIGHT1);
	GLfixed lightPosition[] = { EGL_FixedFromInt(-44), EGL_FixedFromInt(200), EGL_FixedFromInt(100), EGL_FixedFromInt(1) };
	glLightxv(GL_LIGHT1, GL_DIFFUSE, materialColor);
	glLightxv(GL_LIGHT1, GL_POSITION, lightPosition);
	//glEnable(GL_RESCALE_NORMAL);
	GLfixed fogColor[] = { EGL_ONE/10, EGL_ONE/10, 0, 0 };
	glFogxv(GL_FOG_COLOR, fogColor);
	glFogx(GL_FOG_MODE, GL_LINEAR);
	glFogx(GL_FOG_END, EGL_FixedFromInt(600));
	glDisable(GL_FOG);
	//glEnable(GL_FOG);

	//glLineWidth(0x10000);

	//glDisable(GL_TEXTURE_2D);
	glEnable(GL_TEXTURE_2D);
	glTexEnvx(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	glEnable(GL_CULL_FACE);
	//glDisable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	//glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	//glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
	glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	// Test Code until we have full OpenGL pipeline
#if USE_BUFFERS
	glBindBuffer(GL_ARRAY_BUFFER, buffers[VERTEX_BUFFER]);
	glVertexPointer(3, GL_FIXED, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, buffers[NORMAL_BUFFER]);
	glNormalPointer(GL_FIXED, 0, 0);
	glBindBuffer(GL_ARRAY_BUFFER, buffers[TEXTURE_BUFFER]);
	glTexCoordPointer(2, GL_FIXED, 0, 0);
#else
	glVertexPointer(3, GL_FIXED, 0, egl_vertices);
	glNormalPointer(GL_FIXED, 0, egl_normals);
	glTexCoordPointer(2, GL_FIXED, 0, egl_textures);
#endif


	//glScissor(30, 30, 100, 100);
	//glEnable(GL_SCISSOR_TEST);
	//glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
	//glDrawArrays(GL_LINE_STRIP, 0, SIZE);
	glDrawArrays(GL_TRIANGLES, 0, SIZE);

	glFinish();
	eglWaitGL();

	DWORD endRaster = GetTickCount();
	//eglSaveSurfaceHM(g_surface, L"image.bmp");
    eglSwapBuffers(display, g_surface);
	DWORD endSwap = GetTickCount();

	int fps = 1000 / (endSwap - startTicks);
	int clear = clearTicks - startTicks;
	int render = endRaster - clearTicks;
	int blit = endSwap - endRaster;

	wchar_t buffer[500];
	wsprintfW(buffer, L"fps=%d c=%3d[ms] r=%3d[ms] s=%3d[ms]", fps, clear, render, blit);
	RECT rect;
	DrawTextW(hdc, buffer, -1, &rect, DT_CALCRECT | DT_LEFT | DT_SINGLELINE);
	rect.top = 20;
	rect.left = 20;
	SetBkColor(hdc, 0);
	SetTextColor(hdc, 0xffffff);
	DrawTextW(hdc, buffer, -1, &rect, DT_LEFT | DT_SINGLELINE);
	EndPaint(hWnd, &ps);
}
